--[[
These stats are meant to be modified for balancing/designing
]]--

local HeroStatsConstants = {}

-- Ranks indexed in the table
HeroStatsConstants.ranks = {
	{ name = "Low", 		max_points = 2 },		-- 0 (lua counts from 1, so iterating over this table with ipair yields 1-11, adjust accordingly)
	{ name = "Average", 	max_points = 5 },		-- 1
	{ name = "Good", 		max_points = 9 },		-- 2
	{ name = "VeryGood", 	max_points = 14 },		-- 3
	{ name = "Excellent", 	max_points = 20 },		-- 4
	{ name = "Superior", 	max_points = 27 },		-- 5
	{ name = "Heroic", 		max_points = 35 },		-- 6
	{ name = "Legendary", 	max_points = 44 },		-- 7
	{ name = "Epic", 		max_points = 54 },		-- 8
	{ name = "Paragon", 	max_points = 65 },		-- 9
	{ name = "Godlike", 	max_points = 9999 },	-- 10
}

-- ( hero_level / level_bonus_interval ) * stat_bonus.class.per_level_interval = stat_level_bonus
HeroStatsConstants.level_bonus_interval = 5

-- blocking_effectiveness has a baseline of 50%
HeroStatsConstants.blocking_effectiveness_base = 0.5

--[[
Table Logic:

BonusName = { StatName = BonusMultiplier, ... }

A lookup is performed on the BonusName within the table, returning a list of relevant stats and their
BonusMultiplier which is multiplied by whatever the stat level is for the callee.

A bonus can be a POINT bonus or RANK bonus, where a POINT bonus returns a multiplier intended to be
multiplied by stat POINT, and a RANK bonus returns a multiplier intended to be multiplied by stat RANK.

ie.

defence = { agility = 1 } is a POINT bonus, each POINT in agility will yield 1 defence.

match_score.green = { stamina = 1, intelligence = 1 } is a RANK bonus, each RANK in stamina or intelligence
will yield 1 extra score when matching green (where score is used for mana/power gain, or skull damage calcs)
]]--
HeroStatsConstants.mana_start_multiplier = 0.5
HeroStatsConstants.mana_earn_multiplier = 1
HeroStatsConstants.mana_max_multiplier = 2

HeroStatsConstants.stat_bonus = {
	-- Generic:
		-- POINT bonuses
		defence_rating = 	{ agility = 1 },
		shield_critical =	{ stamina = 1 },
		minigame_bonus_turns =	{ intelligence = 1 },
		spell_resist =		{ morale = 1 },
		spell_penetration =	{ intelligence = 1 },
		weapon_critical =	{ strength = 1 },
		
		-- RANK bonuses
		health_per_level = 	{ stamina = 1 },
		weapon_damage = 	{ morale = 1 },
		mana = {
			start = {
				black = 	{ morale = HeroStatsConstants.mana_start_multiplier },
				blue = 		{ intelligence = HeroStatsConstants.mana_start_multiplier },
				green = 	{ stamina = HeroStatsConstants.mana_start_multiplier },
				red = 		{ strength = HeroStatsConstants.mana_start_multiplier },
				yellow = 	{ agility = HeroStatsConstants.mana_start_multiplier },
				power =		{},
			},
			maximum = {
				black = 	{ morale = HeroStatsConstants.mana_max_multiplier },
				blue = 		{ intelligence = HeroStatsConstants.mana_max_multiplier },
				green = 	{ stamina = HeroStatsConstants.mana_max_multiplier },
				red = 		{ strength = HeroStatsConstants.mana_max_multiplier },
				yellow = 	{ agility = HeroStatsConstants.mana_max_multiplier },
			},
		},
		match_score = {
			black =			{ morale = HeroStatsConstants.mana_earn_multiplier },
			blue =			{ intelligence = HeroStatsConstants.mana_earn_multiplier },
			green = 		{ stamina = HeroStatsConstants.mana_earn_multiplier },
			power = 		{ agility = HeroStatsConstants.mana_earn_multiplier },
			red =			{ strength = HeroStatsConstants.mana_earn_multiplier },
			skull = 		{ strength = HeroStatsConstants.mana_earn_multiplier }, -- affects damage
			skull5 = 		{ strength = HeroStatsConstants.mana_earn_multiplier }, -- affects damage
			yellow = 		{ agility = HeroStatsConstants.mana_earn_multiplier },
			-- These are deliberately stubbed, there is no stat based match_score bonus for these gem types,
			-- however the accessor convention of iterating of the returned subtable means that we want to
			-- catch invalid gem type specifications (easier to catch typos that way), so we create stubs for
			-- all types and if a type is requested that isnt in this list, we return nil, which flags an error.
			gold =				{},
			gold5 =				{},
			gold50 =			{},
			gold500 =			{},
			experience =		{},
			empty =				{},
			blocking =			{},
			emptyblocking =		{},
			stealthred =		{},
			stealthgreen =		{},
			stealthblue =		{},
			stealthyellow =		{},
			stealthblack =		{},
			aliveknockred =		{},
			aliveknockgreen =	{},
			aliveknockblue =	{},
			aliveknockyellow =	{},
			knockred =			{},
			knockgreen =		{},
			knockblue =			{},
			knockyellow =		{},
			knockblack =		{},
			wildcard2 =			{},
			wildcard3 =			{},
			wildcard4 =			{},
			wildcard5 =			{},
			wildcard6 =			{},
			wildcard7 =			{},
			chest1 =			{},
			chest2 =			{},
			chest3 =			{},
			chest4 =			{},
			chest5 =			{},
			lock1 =				{},
			lock2 =				{},
			lock3 =				{},
			lock4 =				{},
			lock5 =				{},
			lock6 =				{},
			lock7 =				{},
			door =				{},
			scroll =			{},
			anvil =				{},
			trap =				{},
			monster =			{},
			ambush =			{},
			lock =				{},
			white =				{},
			brown =				{},
		},
	
	-- Class Specific:
		assassin = {
			-- POINT bonus
			start = {
				agility = 1,
				intelligence = 1,
				morale = 1,
				stamina = 1,
				strength = 1,
			},
			per_level_interval = {
				agility = 0,
				intelligence = 0,
				morale = 0,
				stamina = 0,
				strength = 0,
			},
		},
		barbarian = {
			-- POINT bonus
			start = {
				agility = 1,
				intelligence = 1,
				morale = 1,
				stamina = 1,
				strength = 1,
			},
			per_level_interval = {
				agility = 0,
				intelligence = 0,
				morale = 0,
				stamina = 0,
				strength = 0,
			},
		},
		inquisitor = {
			-- POINT bonus
			start = {
				agility = 1,
				intelligence = 1,
				morale = 1,
				stamina = 1,
				strength = 1,
			},
			per_level_interval = {
				agility = 0,
				intelligence = 0,
				morale = 0,
				stamina = 0,
				strength = 0,
			},
		},
		warmage = {
			-- POINT bonus
			start = {
				agility = 1,
				intelligence = 1,
				morale = 1,
				stamina = 1,
				strength = 1,
			},
			per_level_interval = {
				agility = 0,
				intelligence = 0,
				morale = 0,
				stamina = 0,
				strength = 0,
			},
		},
		
		-- Generic catch all for non-class heroes
		class = {
			-- POINT bonus
			start = {
				agility = 0,
				intelligence = 0,
				morale = 0,
				stamina = 0,
				strength = 0,
			},
			per_level_interval = {
				agility = 0,
				intelligence = 0,
				morale = 0,
				stamina = 0,
				strength = 0,
			},
		},
}

return HeroStatsConstants
