
-- THIS FILE NEEDS DOCUMENTATION
-- THIS FILE NEEDS UNIT TESTS

local DEBUG_LOG = function () end
local DEBUG_LOGF = function () end

-- Removed until PSP can divide by zero!
--local POSITIVE_INFINITE = 1.0 / 0.0
local EXCESSIVE_SLEEP_WARNING = 10*60*1000 -- 10 minutes

local METATABLE_FOR_CHILDREN_TABLE = {__mode="k"}
local METATABLE_FOR_CHILDREN_TABLES = {__mode="kv"}

local nextThreadID = 1

if false then
	DEBUG_LOG = LOG
	DEBUG_LOGF = LOGF
end

local Scheduler = {}

function table.count(t)
	local num = 0
	for _,_ in pairs(t) do
		num = num + 1
	end
	return num
end

function table.removevalue(tab, val)
	for i,v in ipairs(tab) do
		if v == val then
			table.remove(tab, i)
			return
		end
	end
end

function Scheduler:Create()
	return Scheduler._Construct({})
end

function Scheduler:_Construct()
	self.newThreads = {}
	self.activeThreads = {}
	self.blockedThreads = {}
	self.blockingThreads = {}			-- threads that were active, but should be moved to pending
	self.pendingThreads = {}			-- threads that were blocked, but should be moved to active
	self.dyingThreads = {}				-- threads that have error conditions, or are finished
	self.stoppingThreads = {}			-- threads that have been passed to StopThread()
	
	self.children = {}					-- keeps track of what owned threads below to what owners
	
	-- Ensure that when the object no longer exists that it does not keep an entry
	-- in this table, otherwise we can never release the objects!
	setmetatable(self.children, METATABLE_FOR_CHILDREN_TABLE)
	
	for k,v in pairs(Scheduler) do
		self[k] = v
	end
	
	return self
end

function Scheduler:AddThread(fn, name)
	assert(fn)

	DEBUG_LOGF("Scheduler:AddThread(%s, %s:%s)", tostring(fn), tostring(nextThreadID), tostring(name))
	
	local thr = {}
	
	thr.start = fn
	thr.name = name or "unknown"
	thr.coro = coroutine.create(fn)
	thr.id = nextThreadID
	
	nextThreadID = nextThreadID + 1
	
	table.insert(self.newThreads, thr)
	
	return thr
end


function Scheduler:AddChildThread(obj, fn, name)
	DEBUG_LOGF("Scheduler:AddChildThread(%s, %s, %s)", tostring(obj), tostring(fn), tostring(name))
	
	local thr = self:AddThread(fn, name)
	if not self.children[obj] then
		self.children[obj] = {}
		setmetatable(self.children[obj], METATABLE_FOR_CHILDREN_TABLES)
	end
	self.children[obj][thr] = thr
	thr.owner = obj
	return thr
end

function Scheduler:StartThread(fn, name)
	self:RunThread(self:AddThread(fn, name))
	self:UpdateThreadLists()
end

function Scheduler:StartChildThread(obj, fn, name)
	self:RunThread(self:AddChildThread(obj, fn, name))
	self:UpdateThreadLists()
end

function Scheduler:StopThread(thr)
	assert(thr)
	
	DEBUG_LOGF(string.format("Scheduler:StopThread(%s:%s)", tostring(thr.id), tostring(thr.name)))

	-- when a thread gets into this table, then it wont be unblocked, nor will it be resumed
	self.stoppingThreads[thr] = thr
end

function Scheduler:StopAllThreadsForObject(obj)
	assert(obj)
	
-- NICK: I had to remove this pring as the tostring() portion was causing 
-- some BIZARRE crashing! See #3650.
--	DEBUG_LOGF(string.format("Scheduler:StopAllThreadsForObject(%s) dumping threads that will be stopped:", tostring(obj)))
	
	if self.children[obj] then
		for thr in pairs(self.children[obj]) do
			self:StopThread(thr)
		end
	end
	
-- See comment above
--	DEBUG_LOGF(string.format("Scheduler:StopAllThreadsForObject(%s) done", tostring(obj)))
end

function Scheduler:RunThread(thr)
	assert(thr)
	
	local err, val 
	
	DEBUG_LOGF("Resuming: %s:%s", tostring(thr.id), tostring(thr.name))
	
	if thr.resumevals then
		err, val = coroutine.resume(thr.coro, unpack(thr.resumevals))
	else
		err, val = coroutine.resume(thr.coro)
	end
	
	DEBUG_LOGF("   resume has finished for %s:%s", tostring(thr.id), tostring(thr.name))
	
	-- only get passed to resume once
	thr.resumevals = nil
	
	if not err then
		error(string.format("thread %s:%s died with error %s", tostring(thr.id), tostring(thr.name), debug.traceback(thr.coro, tostring(val))))
	end
	
	if not err or coroutine.status(thr.coro) == "dead" then
		DEBUG_LOGF("SCHEDULER: thread died of natural causes")
		thr.retval = val
		table.insert(self.dyingThreads, thr)
	elseif val then -- blocking condition received
		thr.condition = val
		table.insert(self.blockingThreads, thr)
	end
end

function Scheduler:Update()

	-- LOGF("Number of children: %d", table.count_pairs(self.children))
	
	-- check all blocked threads
    DEBUG_LOGF("Scheduler:Update(blockedThreadCount = %d)", table.getn(self.blockedThreads))
	
	for index, thr in ipairs(self.blockedThreads) do
		if not self.stoppingThreads[thr] then
			thr.resumevals = { thr.condition() }
			if thr.resumevals[1] then
				table.insert(self.pendingThreads, thr)
			end
		end
	end
	
	-- move all threads that are pending into active from blocked
	for index, thr in pairs(self.pendingThreads) do
		table.insert(self.activeThreads, thr)
		table.removevalue(self.blockedThreads, thr)
	end
	
	-- remove any stopping but blocked threads from the blocked list
	for thr in pairs(self.stoppingThreads) do
		table.removevalue(self.blockedThreads, thr)
	end
	
	-- clear pending
	clear(self.pendingThreads)
	
	-- run all active threads
	DEBUG_LOGF("Scheduler:Update(activeThreadCount = %d)", table.getn(self.activeThreads))
	
	for index, thr in ipairs(self.activeThreads) do
		if not self.stoppingThreads[thr] then
			self:RunThread(thr)
		end
	end
	
	self:UpdateThreadLists()
end

function Scheduler:UpdateThreadLists()
	-- remove and stopping threads present in ANY list
	for thr in pairs(self.stoppingThreads) do
		table.removevalue(self.newThreads, thr)
		table.removevalue(self.activeThreads, thr)
		table.removevalue(self.blockedThreads, thr)
		table.removevalue(self.blockingThreads, thr)
		table.removevalue(self.pendingThreads, thr)
		table.removevalue(self.dyingThreads, thr)
	end
	
	-- move all blocking threads from active to blocked
	for index, thr in pairs(self.blockingThreads) do
		table.insert(self.blockedThreads, thr)
		table.removevalue(self.activeThreads, thr)
		table.removevalue(self.newThreads, thr)
	end
	
	-- remove all dead threads
	for index, thr in pairs(self.dyingThreads) do
		table.removevalue(self.activeThreads, thr)
		table.removevalue(self.newThreads, thr)
	end
	
	-- move all new threads from new to active
	for index, thr in pairs(self.newThreads) do
		table.insert(self.activeThreads, thr)
	end

	-- for any dying threads, remove them from their owner tables
	for index, thr in pairs(self.dyingThreads) do
		if thr.owner then
			self.children[thr.owner][thr] = nil
		end
	end
	
	-- for any stopping threads, remove them from their owner tables
	for thr in pairs(self.stoppingThreads) do
		if thr.owner then
			self.children[thr.owner][thr] = nil
		end
	end
	
	-- clear blocking, dying, stopping
	clear(self.blockingThreads)
	clear(self.dyingThreads)
	clear(self.stoppingThreads)
	clear(self.newThreads)
end

function Scheduler:Sleep(period)
-- Removed until PSP can divide by zero!
--	assert(period ~= POSITIVE_INFINITE, "There is nothing to be gained by sleep forever! (you probably divideded by zero again...)")
	if period >= EXCESSIVE_SLEEP_WARNING then
		LOGF("Warning: you are sleeping a thread for a period of %s ms, is this intentional?", tostring(period))
	end
	local startTime = GetGameTime()
	if period > 800 then
		period = 800
	end
	coroutine.yield(function ()
		return GetGameTime() >= (startTime + period)
	end)
end

function Scheduler:ClearAllThreads()
	DEBUG_LOG("Scheduler:ClearAllThreads()")
	
	-- Stop all our children thread objects
	for object, _ in pairs(self.children) do
		self:StopAllThreadsForObject(object)
	end

	-- Stop all our other threads
	for _, thr in pairs(self.newThreads) do
		self:StopThread(thr)
	end
	for _, thr in pairs(self.activeThreads) do
		self:StopThread(thr)
	end
	for _, thr in pairs(self.blockedThreads) do
		self:StopThread(thr)
	end
	for _, thr in pairs(self.blockingThreads) do
		self:StopThread(thr)
	end
	for _, thr in pairs(self.pendingThreads) do
		self:StopThread(thr)
	end
	for _, thr in pairs(self.dyingThreads) do
		self:StopThread(thr)
	end
	
	DEBUG_LOG("Scheduler:ClearAllThreads() done")
end

return Scheduler
