use_safeglobals()

local ProbabilityTable = require("Utils/ProbabilityTable")

class "BattleGameMenu" (cGameMenu);

local HINT_DIST = {
	["[HINT_SPELL_X_READY]"] = 1,
	["[HINT_ITEM_X_READY]"]  = 1,
	["[HINT_DEFAULT1]"]      = 1,
	["[HINT_DEFAULT2]"]      = 1,
}

local default_hints = {
	"[HINT_DEFAULT1]",
	"[HINT_DEFAULT2]",
}

-- Any platform specific stuff in here
dofile("Assets/Scripts/Screens/BattleGameMenuPlatform.lua")

local MODE_GRID  = 0
local MODE_LEFT  = 1
local MODE_RIGHT = 2

local MAX_SPELLS = 5

local LuaGamepad = require("Controls.Gamepad")
local gp_mode = { location = MODE_GRID, index = "spell1", show_info = false }
local buttons_pressed = {}
local gp_navigation = {
	status_effects = { widget="pad_statuseffect", index = 1, up="spell5", down="portrait", pad=true, },
	
	portrait = { widget="pad_portrait", up="status_effects", down="weapon1", pad=true, },
	
	weapon1 = { widget="butt_weapon1", up="portrait",  down="item1",  right="weapon2", left="weapon2" },
	weapon2 = { widget="butt_weapon2", up="portrait",  down="item3",  right="weapon1", left="weapon1" },
	
	item1    = { widget="pad_item1", up="weapon1", down="spell1", right="item2",    left="item4",  pad=true, },
	item2    = { widget="pad_item2", up="weapon1", down="spell1", right="item3",    left="item1",  pad=true, },
	item3    = { widget="pad_item3", up="weapon2", down="spell1", right="item4",    left="item2",  pad=true, },
	item4    = { widget="pad_item4", up="weapon2", down="spell1", right="item1",    left="item3",  pad=true, },
	
	spell1  = { widget="butt_spell1", up="item1",   down="spell2"         },
	spell2  = { widget="butt_spell2", up="spell1",  down="spell3"         },
	spell3  = { widget="butt_spell3", up="spell2",  down="spell4"         },
	spell4  = { widget="butt_spell4", up="spell3",  down="spell5"         },
	spell5  = { widget="butt_spell5", up="spell4",  down="status_effects" },
}
local function GetGamepadWidget(area)
	if area == "status_effects" then
		return string.format("%s%d", gp_navigation[area].widget, gp_navigation[area].index)
	else
		return gp_navigation[area].widget
	end
end


function BattleGameMenu:__init()
	super()
	
	self:InitPlatform()
	self:Initialize("Assets\\Screens\\BattleGameMenu.xml")
	
	SoundSystem.Load("board_bank")
	
	self.view = GameObjectManager:ConstructLocal("BGND")
	self.view.host = self.host
	self.clients = {}
	
	_G.Joystick.InitJoystick()
	
	self:SetWorld(self.view)
end

function BattleGameMenu:OnOpen()
	-- The EffectsMenu is a transparent screen that sits over all other menus so that we
	-- can put FX containers and particle systems in it and they will appear over all 
	-- other UI components.  
	
	self:AddNetworkHandlers()
	self:OpenPlatform()
	
	SCREENS.EffectsMenu:Open()
	SCREENS.EffectsMenu:MoveToFront()
	
	-- Note: here we want the windows (back to front):
	--    BackdropMenu
	--    BattleGameMenuBackground
	--    PlayerInfoMenu (x2)
	--    BattleGameMenu
	--    EffectsMenu
	-- So we need to move BackdropMenu back to the back after opening
	-- and moving BattleGameMenuBackground to the back
	
	SCREENS.BattleGameMenuBackground:Open()
	SCREENS.BattleGameMenuBackground:MoveToBack()
	if IsOpen("BackdropMenu") then
		SCREENS.BackdropMenu:MoveToBack()
	end
	
	buttons_pressed = { }
	
--	MenuSystem.SetGamepadWidget(self, "BattleGameMenuWorldPad")
	
	self.hintdelay = Settings:Read("hintarrowdelay", 5000)
	
	if IsGamepadActive() then
		self.show_info = false
		--self:set_text("str_gp_y", "[MORE_INFO_SHOW]")
	end
	
	-- Update Music
	MusicManager:SetState("BATTLE_START")
	self:Preload()
	
	return cGameMenu.OnOpen(self)
end

function BattleGameMenu:OnRefresh()
	self:hide_widget("str_cancelhelptext")
	
	self.view:RefreshView()
end

function BattleGameMenu:RefreshSpellTargetMenu(client)
	assert(client)
	
	local x, y = SCREENS[client]:CalculateSpellTargetMenuPos()
	SCREENS.SpellTargetMenu:Move(x, y)
end

function BattleGameMenu:RefreshClientMenus()
	for index, client in ipairs(self.clients) do
		local x, y = self:GetInfoLocations(index)
		if x ~= SCREENS[client]:GetX() and y ~= SCREENS[client]:GetY() then
			SCREENS[client]:Move(x, y)
		end
		
		self:SetupPlayerInfo(index, client)
	end
end

function BattleGameMenu:OnClose()
	LOG("BattleGameMenu:OnClose")
	require("FXContainer").StopAll()

	self:ClosePlatform()
	self:RemoveNetworkHandlers()
	
	-- contains the background images for this menu
	SCREENS.BattleGameMenuBackground:Close()
	
	-- make sure all client windows are closed
	for i, client in ipairs(self.clients) do
		SCREENS[client]:Close()
		SCREENS[client] = nil
	end
	
	self:HideStealthHalo()
	
	GameObjectManager:Destroy(self.view)
	self.view = nil
	self:SetWorld(nil)
	
	scheduler:StopAllThreadsForObject(self)
	
	if IsOpen("OppTutorialMenu") then
		SCREENS.OppTutorialMenu:Close()
	end
	
	SCREENS.EffectsMenu:DestroyEffects()
	
	SoundSystem.Unload("board_bank")
	
	return cGameMenu.OnClose(self)
end

function BattleGameMenu:IsLocalMultiplayer()
	return self.clients[1]:GetAttribute("local") and self.clients[2]:GetAttribute("local") and not self.clients[1]:GetAttribute("npc") and not self.clients[2]:GetAttribute("npc")
end

function BattleGameMenu:OnKey(key)
	if key == self.CHEAT_LEFT_WIN then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			-- automatically win game, can only do this if we have a host object
			if Battles.IsHost() then
				Battles.GetHost():CheatBattleWin()
			end
		end
	elseif key == self.CHEAT_RIGHT_WIN then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			-- automatically lose game, can only do this if we have a host object
			if Battles.IsHost() then
				Battles.GetHost():CheatBattleLose()
			end
		end
	elseif key == self.CHEAT_PREV_BOARD then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			if Battles.IsHost() then
				Battles.GetHost():CheatPreviousBoardLayout()
			end
		end
	elseif key == self.CHEAT_NEXT_BOARD then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			if Battles.IsHost() then
				Battles.GetHost():CheatNextBoardLayout()
			end
		end
	elseif key == self.CHEAT_LEFT_MORE_MANA then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			if Battles.IsHost() then
				Battles.GetHost():CheatLeftManaGain()
			end
		end
	elseif key == self.CHEAT_RIGHT_MORE_MANA then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			if Battles.IsHost() then
				Battles.GetHost():CheatRightManaGain()
			end
		end
	elseif key == self.CHEAT_LEFT_MORE_HEALTH then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			if Battles.IsHost() then
				Battles.GetHost():CheatLeftHealthGain()
			end
		end
	elseif key == self.CHEAT_RIGHT_MORE_HEALTH then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			if Battles.IsHost() then
				Battles.GetHost():CheatRightHealthGain()
			end
		end
	elseif key == self.CHEAT_LEFT_BUMP_MANA_UP then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			if Battles.IsHost() then
				Battles.GetHost():CheatLeftManaAdd(1)
			end
		end
	elseif key == self.CHEAT_LEFT_BUMP_MANA_DOWN then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			if Battles.IsHost() then
				Battles.GetHost():CheatLeftManaAdd(-1)
			end
		end
	elseif key == self.DEBUG_COLLECTGARBAGE then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			collectgarbage()
		end
	elseif key == self.QUIT_KEY then
		if IsPlatformPCorSTEAM() then
			self:OpenPauseMenu()
		end		
	-- And a whole bunch of keys that change gems
	elseif key == string.byte("r") or key == string.byte("R") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost()then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Red")
			end
		end
	elseif key == string.byte("g") or key == string.byte("G") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Green")
			end
		end
	elseif key == string.byte("b") or key == string.byte("B") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Blue")
			end
		end
	elseif key == string.byte("y") or key == string.byte("Y") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Yellow")
			end
		end
	elseif key == string.byte("p") or key == string.byte("P") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Black")
			end
		end
	elseif key == string.byte("a") or key == string.byte("A") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Power")
			end
		end
	elseif key == string.byte("s") or key == string.byte("S") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Skull")
			end
		end
	elseif key == string.byte("d") or key == string.byte("D") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Skull5")
			end
		end
	elseif key == string.byte("2") or key == string.byte("@") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Wildcard2")
			end
		end
	elseif key == string.byte("3") or key == string.byte("#") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Wildcard3")
			end
		end
	elseif key == string.byte("4") or key == string.byte("$") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Wildcard4")
			end
		end
	elseif key == string.byte("5") or key == string.byte("%") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Wildcard5")
			end
		end
	elseif key == string.byte("6") or key == string.byte("^") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Wildcard6")
			end
		end
	elseif key == string.byte("7") or key == string.byte("&") then
		if not IsComputerBuildslaveAndBuildConfigRetail() then
			local selectedGem = self:GetWorld():GetGemUnderCursor()
			if selectedGem and Battles.IsHost() then
				Battles.GetHost():CheatCycleGem(selectedGem.x, selectedGem.y, "Wildcard7")
			end
		end
	end
	
	return Menu.OnKey(self, key)
end

function BattleGameMenu:SetGameInfo(layout, gems, numClients)
--[[
	THIS CODE NEEDS TO BE REPLACED/FIXED LATER (Nick)

	-- use the width and height to set the background image
	if layout.width == 8 and layout.height == 8 then
		self:set_image("icon_board", "img_board8")
	elseif layout.width == 12 and layout.height == 12 then
		self:set_image("icon_board", "img_board12")
	elseif layout.width == 16 and layout.height == 16 then
		self:set_image("icon_board", "img_board16")
	end
]]
end

function BattleGameMenu:SetGameName(name)
	self:set_text("str_heading", name)
end

function BattleGameMenu:SetClientInfo(index, client)
	LOGF("Adding PlayerInfoMenu for: " .. tostring(client.hero))
	
	SCREENS[client] = require("Screens/PlayerInfoMenu")(index)
	SCREENS[client].client = client
	SCREENS[client].hero = client.hero
	SCREENS[client]:Open(self:GetInfoLocations(index))
	
	self:SetupPlayerInfo(index, client)
	
	-- make sure the BattleGameMenu and EffectsMenu always sits on top
	SCREENS.BattleGameMenu:MoveToFront()
	SCREENS.EffectsMenu:MoveToFront()
	
	table.insert(self.clients, client)
end

function BattleGameMenu:SetupPlayerInfo(index, client)
	if index == 2 then
		SCREENS[client]:set_image_flip("icon_portrait", true)
		SCREENS[client]:set_image_flip("icon_turn_top", true)
		
		SCREENS[client]:set_image("icon_weapon1_ready", "img_weaponbutton_ready_player2")
		SCREENS[client]:set_image("icon_weapon2_ready", "img_weaponbutton_ready_player2")
		
		local offset = GetScreenOffset()
		if offset == 0 then	-- not widescreen
			-- reset gp_navigation
			gp_navigation.status_effects = { widget="pad_statuseffect", index = 1, up="spell5", down="portrait", pad=true, }
			gp_navigation.portrait = { widget="pad_portrait", up="status_effects", down="weapon1", pad=true, }
			gp_navigation.weapon1 = { widget="butt_weapon1", up="portrait", down="item1", right="weapon2", left="weapon2" }
			gp_navigation.weapon2 = { widget="butt_weapon2", up="portrait", down="item3", right="weapon1", left="weapon1" }
			gp_navigation.item1 = { widget="pad_item1", up="weapon1", down="spell1", right="item2", left="item4", pad=true, }
			gp_navigation.item2 = { widget="pad_item2", up="weapon1", down="spell1", right="item3", left="item1", pad=true, }
			gp_navigation.item3 = { widget="pad_item3", up="weapon2", down="spell1", right="item4", left="item2", pad=true, }
			gp_navigation.item4 = { widget="pad_item4", up="weapon2", down="spell1", right="item1", left="item3", pad=true, }
			gp_navigation.spell1 = { widget="butt_spell1", up="item1",   down="spell2" }
			
			local icon_timer_pos_x, icon_timer_pos_y = self:GetWidgetLocations("icon_timer")
			local str_timer_pos_x, str_timer_pos_y  = self:GetWidgetLocations("str_timer_value")
			SCREENS[client]:set_widget_position("icon_timer", icon_timer_pos_x, icon_timer_pos_y)
			SCREENS[client]:set_widget_position("str_timer_value", str_timer_pos_x, str_timer_pos_y)
			
			local turn_flame_pos_x, turn_flame_pos_y = self:GetWidgetLocations("icon_turn_flame")
			local turn_top_pos_x, turn_top_pos_y  = self:GetWidgetLocations("icon_turn_top")
			SCREENS[client]:set_widget_position("icon_turn_flame", turn_flame_pos_x, turn_flame_pos_y)
			SCREENS[client]:set_widget_position("icon_turn_top", turn_top_pos_x, turn_top_pos_y)
			
			for i=1, 5 do
				SCREENS[client]:set_image(string.format("icon_spell%d_ready", i), string.format("img_spellbutton%d_ready_player2", i))
			end
			
			for j=1, 10 do
				local x = SCREENS[client]:get_widget_x(string.format("icon_statuseffect%d", j))
				local y = SCREENS[client]:get_widget_y(string.format("icon_statuseffect%d", j))
				
				local statuseffect_1_pos_x, statuseffect_1_pos_y = self:GetWidgetLocations("icon_statuseffect_1")
				local statuseffect_2_pos_x, statuseffect_2_pos_y = self:GetWidgetLocations("icon_statuseffect_2")
				local statuseffect_3_pos_x, statuseffect_3_pos_y = self:GetWidgetLocations("icon_statuseffect_3")
				
				local new_x = x
				if j < 5 then
					new_x = x + statuseffect_1_pos_x
				elseif j < 9 then
					new_x = x + statuseffect_2_pos_x
				else
					new_x = x + statuseffect_3_pos_x
				end
				
				SCREENS[client]:set_widget_position(string.format("icon_statuseffect%d", j), new_x, y)
				SCREENS[client]:set_widget_position(string.format("pad_statuseffect%d", j), new_x, y)
				SCREENS[client]:set_widget_position(string.format("icon_statuseffect%d_highlight", j), new_x, y)
				SCREENS[client]:set_widget_position(string.format("remainingturn_statuseffect%d", j), new_x, y)
			end
			
			local att_x = SCREENS[client]:get_widget_x("icon_attackrating")
			local att_y = SCREENS[client]:get_widget_y("icon_attackrating")
			
			local attackrating_pos_x, attackrating_pos_y = self:GetWidgetLocations("icon_attackrating")
			
			local new_att_x = att_x - attackrating_pos_x
			
			SCREENS[client]:set_widget_position("icon_attackrating", new_att_x, att_y)
			SCREENS[client]:set_widget_position("pad_attackrating", new_att_x, att_y)
			SCREENS[client]:set_widget_position("str_attack_value", new_att_x, att_y)
			
			local def_x = SCREENS[client]:get_widget_x("icon_defrating")
			local def_y = SCREENS[client]:get_widget_y("icon_defrating")
			
			local defrating_pos_x, defrating_pos_y = self:GetWidgetLocations("icon_defrating")
			
			local new_def_x = def_x - defrating_pos_x
			
			SCREENS[client]:set_widget_position("icon_defrating", new_def_x, def_y)
			SCREENS[client]:set_widget_position("pad_defrating", new_def_x, def_y)
			SCREENS[client]:set_widget_position("str_def_value", new_def_x, def_y)
		else
			-- set gp_navigation for widescreen
			gp_navigation.status_effects = { widget="pad_statuseffect", index = 1, up="spell5", down="portrait", pad=true, }
			gp_navigation.portrait = { widget="pad_portrait", up="status_effects", down="weapon1", pad=true, }
			gp_navigation.weapon1 = { widget="butt_weapon1", up="portrait", down="spell1", right="weapon2", left="item1" }
			gp_navigation.weapon2 = { widget="butt_weapon2", up="portrait", down="spell1", right="item1", left="weapon1" }
			gp_navigation.item1 = { widget="pad_item1", up="portrait", down="item2", right="weapon1", left="weapon2", pad=true, }
			gp_navigation.item2 = { widget="pad_item2", up="item1", down="item3", right="weapon1", left="weapon2", pad=true, }
			gp_navigation.item3 = { widget="pad_item3", up="item2", down="item4", right="weapon1", left="weapon2", pad=true, }
			gp_navigation.item4 = { widget="pad_item4", up="item3", down="spell1", right="weapon1", left="weapon2", pad=true, }
			gp_navigation.spell1 = { widget="butt_spell1", up="item4", down="spell2" }
			
			local icon_timer_pos_x, icon_timer_pos_y = self:GetWidgetLocations("icon_timer")
			local str_timer_pos_x, str_timer_pos_y  = self:GetWidgetLocations("str_timer_value")
			SCREENS[client]:set_widget_position("icon_timer", icon_timer_pos_x, icon_timer_pos_y)
			SCREENS[client]:set_widget_position("str_timer_value", str_timer_pos_x, str_timer_pos_y)
			
			local turn_flame_pos_x, turn_flame_pos_y = self:GetWidgetLocations("icon_turn_flame")
			SCREENS[client]:set_widget_position("icon_turn_flame", turn_flame_pos_x, turn_flame_pos_y)
			
			for i=1, 5 do
				SCREENS[client]:set_image(string.format("icon_spell%d_ready", i), string.format("img_spellbutton%d_ready_player2_ws", i))
			end
			
			for j=1, 10 do
				local x = SCREENS[client]:get_widget_x(string.format("icon_statuseffect%d", j))
				local y = SCREENS[client]:get_widget_y(string.format("icon_statuseffect%d", j))
				
				local statuseffect_1_pos_x, statuseffect_1_pos_y = self:GetWidgetLocations("icon_statuseffect_1")
				local statuseffect_2_pos_x, statuseffect_2_pos_y = self:GetWidgetLocations("icon_statuseffect_2")
				local statuseffect_3_pos_x, statuseffect_3_pos_y = self:GetWidgetLocations("icon_statuseffect_3")
				
				local new_x = x
				if j < 5 then
					new_x = x + statuseffect_1_pos_x
				elseif j < 9 then
					new_x = x + statuseffect_2_pos_x
				else
					new_x = x + statuseffect_3_pos_x
				end
				
				SCREENS[client]:set_widget_position(string.format("icon_statuseffect%d", j), new_x, y)
				SCREENS[client]:set_widget_position(string.format("pad_statuseffect%d", j), new_x, y)
				SCREENS[client]:set_widget_position(string.format("icon_statuseffect%d_highlight", j), new_x, y)
				SCREENS[client]:set_widget_position(string.format("remainingturn_statuseffect%d", j), new_x, y)
			end
			
			local att_x = SCREENS[client]:get_widget_x("pad_attackrating")
			local att_y = SCREENS[client]:get_widget_y("pad_attackrating")
			
			local attackrating_pos_x, attackrating_pos_y = self:GetWidgetLocations("icon_attackrating")
			local new_att_x = att_x - attackrating_pos_x
			
			SCREENS[client]:set_widget_position("icon_attackrating", new_att_x, att_y)
			SCREENS[client]:set_widget_position("pad_attackrating", new_att_x, att_y)
			SCREENS[client]:set_widget_position("str_attack_value", new_att_x, att_y)
			
			local def_x = SCREENS[client]:get_widget_x("pad_defrating")
			local def_y = SCREENS[client]:get_widget_y("pad_defrating")
			
			local defrating_pos_x, defrating_pos_y = self:GetWidgetLocations("icon_defrating")	-- icon_defrating does not actually exist as a widget, these are just hardcoded values
			
			local new_def_x = def_x - defrating_pos_x
			
			SCREENS[client]:set_widget_position("pad_defrating", new_def_x, def_y)
			SCREENS[client]:set_widget_position("str_def_value", new_def_x, def_y)
			
			for k=1, 4 do
				local x = SCREENS[client]:get_widget_x(string.format("pad_item%d", k))
				local y = SCREENS[client]:get_widget_y(string.format("pad_item%d", k))
				
				local item_pos_x, item_pos_y = self:GetWidgetLocations("icon_item")
				
				local new_x = x + item_pos_x
				
				SCREENS[client]:set_widget_position(string.format("pad_item%d", k), new_x, y)
				SCREENS[client]:set_widget_position(string.format("icon_item%d", k), new_x, y)
				SCREENS[client]:set_widget_position(string.format("icon_item%d_overlay", k), new_x, y)
				SCREENS[client]:set_widget_position(string.format("icon_item%d_highlight", k), new_x, y)
			end
			
			local pwr_x = SCREENS[client]:get_widget_x("str_power_value")
			local pwr_y = SCREENS[client]:get_widget_y("str_power_value")
			
			local str_power_x, str_power_y = self:GetWidgetLocations("str_power")
			
			local new_pwr_x = pwr_x - str_power_x
			
			SCREENS[client]:set_widget_position("str_power_value", new_pwr_x, pwr_y)
			
			for l=1, 2 do
				local x = SCREENS[client]:get_widget_x(string.format("icon_weapon%d", l))
				local y = SCREENS[client]:get_widget_y(string.format("icon_weapon%d", l))
				local icon_weapon_x, icon_weapon_y = self:GetWidgetLocations(string.format("icon_weapon%d", l))
				
				local new_x = x - icon_weapon_x
				
				SCREENS[client]:set_widget_position(string.format("icon_weapon%d", l), new_x, y)
				SCREENS[client]:set_widget_position(string.format("icon_weapon%d_overlay", l), new_x, y)
				SCREENS[client]:set_widget_position(string.format("icon_weapon%d_highlight", l), new_x, y)
				
				x = SCREENS[client]:get_widget_x(string.format("icon_weapon%d_ready", l))
				y = SCREENS[client]:get_widget_y(string.format("icon_weapon%d_ready", l))
				new_x = x - icon_weapon_x
				
				SCREENS[client]:set_widget_position(string.format("icon_weapon%d_ready", l), new_x, y)
				
				x = SCREENS[client]:get_widget_x(string.format("butt_weapon%d", l))
				y = SCREENS[client]:get_widget_y(string.format("butt_weapon%d", l))
				new_x = x - icon_weapon_x
				
				SCREENS[client]:set_widget_position(string.format("butt_weapon%d", l), new_x, y)
			end
			
			local mana_colours = {"black", "blue", "yellow", "red", "green"}
			for m=1, 5 do
				local x = SCREENS[client]:get_widget_x(string.format("icon_%s", mana_colours[m]))
				local y = SCREENS[client]:get_widget_y(string.format("icon_%s", mana_colours[m]))
				
				local icon_mana_x, icon_mana_y = self:GetWidgetLocations("icon_mana")
				
				local new_x = x - icon_mana_x				
				
				SCREENS[client]:set_widget_position(string.format("icon_%s", mana_colours[m]), new_x, y)
				SCREENS[client]:set_widget_position(string.format("icon_%s_filling", mana_colours[m]), new_x, y)
				
				x = SCREENS[client]:get_widget_x(string.format("str_%s_value", mana_colours[m]))
				y = SCREENS[client]:get_widget_y(string.format("str_%s_value", mana_colours[m]))
				new_x = x - icon_mana_x
				
				SCREENS[client]:set_widget_position(string.format("str_%s_value", mana_colours[m]), new_x, y)
			end
		end
	end
end

-- returns state of targettingLockdown flag for both clients (eg in targetting mode)
--    0 = neither client locked
--    1 = client 1 is locked but not client 2
--    2 = client 2 is locked but not client 1
--    3 = both clients are locked down (this would mean both clients are in targetting mode so it shouldn't ever happen)
function BattleGameMenu:ClientLockedDown()
	local return_value = 0
	for i = 1, 2 do
		if SCREENS[self.clients[i]].targettingLockdown then
			return_value = return_value + i
		end
	end
	return return_value
end

function BattleGameMenu:OnButton(buttonId, clickX, clickY)
	return cGameMenu.OnButton(self, buttonId, clickX, clickY)
end

function BattleGameMenu:OnMouseRightButton()
	for _, client in ipairs(self.clients) do	-- HAX!
		SCREENS[client]:DisableTargettingLockdown()
	end
	
	return Menu.MESSAGE_HANDLED
end

function BattleGameMenu:OnGamepadSlider(user_id, slider_id, value)
	if (value > 96) or (value < -96) then
		if self.slider_deadzoned then
			if slider_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_SLIDER_LEFT) then
				if self:GetWorld().allowMoves and self:ClientLockedDown() == 0 then
					if gp_mode.location == MODE_GRID then
						gp_mode.location = MODE_LEFT
						self.selected = false
						self.view:Unselect()
						self:MoveToUI("spell1")
					elseif gp_mode.location == MODE_LEFT or gp_mode.location == MODE_RIGHT then
						self:MoveToGrid()
					end
				end
			elseif slider_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_SLIDER_RIGHT) then
				if self:GetWorld().allowMoves and self:ClientLockedDown() == 0 then
					if gp_mode.location == MODE_GRID then
						gp_mode.location = MODE_RIGHT
						self.selected = false
						self.view:Unselect()
						self:MoveToUI("spell1")
					elseif gp_mode.location == MODE_LEFT or gp_mode.location == MODE_RIGHT then
						self:MoveToGrid()
					end
				end
			end
		end
		self.slider_deadzoned = false
	else
		self.slider_deadzoned = true
	end
	return Menu.MESSAGE_HANDLED
end

function BattleGameMenu:OnGamepadButton(user_id, button_id, value)
	LOGF("BattleGameMenu:OnGamepadButton(user_id=%d, button_id=%d, value=%d)", user_id, button_id, value)
	
	if value == 0 then
		buttons_pressed[button_id] = false
		return Menu.MESSAGE_HANDLED
	end
	
	buttons_pressed[button_id] = true
	if buttons_pressed[GamepadInput.XBOX360_BUTTON_BACK] or buttons_pressed[GamepadInput.PC_BUTTON_7] then
		if buttons_pressed[Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_LEFT)] then
			self:OnKey(self.CHEAT_LEFT_MORE_MANA)
		elseif buttons_pressed[Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_TOP)] then
			self:OnKey(self.CHEAT_RIGHT_MORE_MANA)
		elseif buttons_pressed[Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_SHOULDER_LEFT)] then
			self:OnKey(self.CHEAT_LEFT_WIN)
		elseif buttons_pressed[Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_SHOULDER_RIGHT)] then
			self:OnKey(self.CHEAT_RIGHT_WIN)
		elseif buttons_pressed[Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_OKAY)] then
			self:OnKey(self.CHEAT_LEFT_MORE_HEALTH)
		elseif buttons_pressed[Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CANCEL)] then
			self:OnKey(self.CHEAT_RIGHT_MORE_HEALTH)
		elseif buttons_pressed[Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_SLIDER_LEFT)] then	-- does not work because on gamepad slide picks these up
			self:OnKey(self.CHEAT_PREV_BOARD)
		elseif buttons_pressed[Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_SLIDER_RIGHT)] then
			self:OnKey(self.CHEAT_NEXT_BOARD)
		end
		return Menu.MESSAGE_HANDLED
	end
	
	if button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_OKAY) then
		if gp_mode.location == MODE_GRID then
			if IsOpen("OppTutorialMenu") then
				SCREENS.OppTutorialMenu:Close()
			end
			
			if not self.selected then
				if self:GetWorld():IsInTargettingMode() then
					self:GetWorld():DoGamepadTarget()
				else
					self.selected = true
					self.view:HideHint()
					self.view:SetSpinSpeed(true)
				end
			else
				self.selected = false
				self.view:ShowHint()
				self.view:SetSpinSpeed(false)
			end
		end
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CANCEL) then
		if self:GetWorld():IsInTargettingMode() then
			for _, client in ipairs(self.clients) do	-- HAX!
				SCREENS[client]:DisableTargettingLockdown()
			end
		elseif gp_mode.location == MODE_GRID then
			if self.selected then
				self.selected = false
				self.view:ShowHint()
				self.view:SetSpinSpeed(false)
			end
		end
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_START) then
		self:OpenPauseMenu(user_id)
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_SHOULDER_LEFT) then
		if self:GetWorld().allowMoves and self:ClientLockedDown() == 0 then
			if gp_mode.location == MODE_GRID then
				gp_mode.location = MODE_LEFT
				self.selected = false
				self.view:Unselect()
				self:MoveToUI("weapon1")
			elseif gp_mode.location == MODE_LEFT or gp_mode.location == MODE_RIGHT then
				self:MoveToGrid()
			end
		end
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_SHOULDER_RIGHT) then
		if self:GetWorld().allowMoves and self:ClientLockedDown() == 0 then
			if gp_mode.location == MODE_GRID then
				gp_mode.location = MODE_RIGHT
				self.selected = false
				self.view:Unselect()
				self:MoveToUI("weapon1")
			elseif gp_mode.location == MODE_LEFT or gp_mode.location == MODE_RIGHT then
				self:MoveToGrid()
			end
		end
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_TOP) then
		gp_mode.show_info = not gp_mode.show_info
		if gp_mode.show_info then
			SCREENS.BattleGameMenuBackground:set_text("str_gp_y", "[MORE_INFO_HIDE]")
		else
			SCREENS.BattleGameMenuBackground:set_text("str_gp_y", "[MORE_INFO_SHOW]")
		end
		
		if gp_mode.location ~= MODE_GRID then
			local widget = gp_navigation[gp_mode.index].widget
			local screen = SCREENS[self.clients[gp_mode.location]]
			if gp_navigation[gp_mode.index].pad then
				if gp_mode.index == "status_effects" then
					widget = string.format("%s%d", widget, gp_navigation[gp_mode.index].index)
					screen:OnMouseEnter(screen:get_widget_id(widget), screen:get_widget_abs_x(widget), screen:get_widget_abs_y(widget))
				else
					screen:OnMouseEnter(screen:get_widget_id(widget), screen:get_widget_abs_x(widget), screen:get_widget_abs_y(widget))
				end
			else
				screen:OnButtonAvailable(screen:get_widget_id(widget), screen:get_widget_abs_x(widget), screen:get_widget_abs_y(widget), true)
			end
			MenuSystem.SetGamepadWidget(screen, widget)
		end
	end
	
	return Menu.MESSAGE_HANDLED
end

function BattleGameMenu:MoveToGrid()
	if(gp_mode.location ~= MODE_GRID) then
		local screen = SCREENS[self.clients[gp_mode.location]]
		screen:hide_widget("icon_portrait_frame_highlight1")
		screen:hide_widget("icon_portrait_frame_highlight2")
		if gp_mode.index == "status_effects" then
			screen:hide_widget(string.format("icon_statuseffect%d_highlight", gp_navigation.status_effects.index))
		else
			if gp_mode.index == "weapon1" or 
			   gp_mode.index == "weapon2" or
				gp_mode.index == "item1" or 
				gp_mode.index == "item2" or
				gp_mode.index == "item3" or
				gp_mode.index == "item4" then
				screen:set_alpha(string.format("icon_%s", gp_mode.index), 0.6)
			end
			screen:hide_widget(string.format("icon_%s_highlight", gp_mode.index))
		end
	end
	gp_mode.location = MODE_GRID
	-- set index to something other then "status_effects" when moving to the grid (#3190)
	gp_mode.index = "spell1"
	-- do NOT set the widget if the pause menu is open (#2822)
	if not IsOpen("PauseMenu") then
		MenuSystem.SetGamepadWidget(self, "pad_arena")
	end
	self.view:SetupSelection()
	close_custompopup_menu()
end

function BattleGameMenu:MoveToUI(dest)
	self:MoveGamepadFocus(dest or "spell1")
end

function BattleGameMenu:MoveGamepadFocus(new_location, dir)
	assert(gp_navigation[new_location], string.format("Gamepad moved to unknown location %s", tostring(new_location)))
	close_custompopup_menu()
	local data = gp_navigation[new_location]
	local screen = SCREENS[self.clients[gp_mode.location]]
	screen:hide_widget(string.format("icon_%s_highlight", gp_mode.index))
	--[[
	do
		local commands = screen:QueryAvailableCommands()
		if string.sub(new_location, 1, 4) == "item" then
			if not commands[string.format("item%s", string.sub(new_location, -1))] then
				self:MoveGamepadFocus(gp_navigation[new_location][dir], dir)
				return
			end
		elseif string.sub(new_location, 1, 5) == "spell" then
			if not commands[string.format("spell%s", string.sub(new_location, -1))] then
				self:MoveGamepadFocus(gp_navigation[new_location][dir], dir)
				return
			end
		elseif new_location == "weapon1" then
			if not commands.weapon1 then
				self:MoveGamepadFocus(gp_navigation[new_location][dir], dir)
				return
			end
		elseif new_location == "weapon2" then
			if not commands.weapon2 then
				self:MoveGamepadFocus(gp_navigation[new_location][dir], dir)
				return
			end
		end
	end
	]]--
	
	local statusEffectArray = {}
	for k,v in pairs(screen.displayedEffects) do
		table.insert(statusEffectArray, v)
	end
	
	if gp_mode.index == "status_effects" then
		
		local old_index = gp_navigation.status_effects.index
		screen:hide_widget(string.format("icon_statuseffect%d_highlight", old_index))
		local new_index
		if dir == "up" then
			new_index = gp_navigation.status_effects.index - 1
			gp_navigation.status_effects.index = new_index
		elseif dir == "down" then
			new_index = gp_navigation.status_effects.index + 1
			gp_navigation.status_effects.index = new_index
		elseif dir == "left" then
			new_index = gp_navigation.status_effects.index - 4
			if new_index > 0 then
				gp_navigation.status_effects.index = new_index
			end
		elseif dir == "right" then
			new_index = gp_navigation.status_effects.index + 4
			if new_index <= #statusEffectArray then
				gp_navigation.status_effects.index = new_index
			end
		else
			gp_navigation.status_effects.index = 1
			screen:activate_widget(string.format("icon_statuseffect1_highlight"))
			return
		end
		
		if statusEffectArray[gp_navigation.status_effects.index] then
			local widget = GetGamepadWidget(gp_mode.index)
			screen:OnMouseEnter(screen:get_widget_id(widget), self:GetX() + screen:get_widget_x(widget), self:GetY() + screen:get_widget_y(widget))
			screen:activate_widget(string.format("icon_statuseffect%d_highlight", new_index))
			return
		end

		for i=1, 10 do
			screen:hide_widget(string.format("icon_statuseffect%d_highlight", i))
		end
	elseif gp_mode.index == "weapon1" or 
	       gp_mode.index == "weapon2" or 
			 gp_mode.index == "item1" or 
			 gp_mode.index == "item2" or 
			 gp_mode.index == "item3" or 
			 gp_mode.index == "item4" then
		screen:set_alpha(string.format("icon_%s", gp_mode.index), 0.6)
	elseif gp_mode.index == "portrait" then
		screen:hide_widget("icon_portrait_frame_highlight1")
		screen:hide_widget("icon_portrait_frame_highlight2")
	end
	
	if new_location == "status_effects" then
		if #statusEffectArray == 0 then
			self:MoveGamepadFocus(gp_navigation[new_location][dir], dir)
			return
		else
			if dir == "up" then
				gp_navigation.status_effects.index = #statusEffectArray
			elseif dir == "down" then
				gp_navigation.status_effects.index = 1
			end
			screen:activate_widget(string.format("icon_statuseffect%d_highlight", gp_navigation.status_effects.index))
		end
	elseif new_location == "weapon1" or 
	       new_location == "weapon2" or 
			 new_location == "item1" or
			 new_location == "item2" or
			 new_location == "item3" or
			 new_location == "item4" then
		screen:set_alpha(string.format("icon_%s", new_location), 1.0)
	elseif new_location == "portrait" then
		screen:activate_widget("icon_portrait_frame_highlight1")
		screen:activate_widget("icon_portrait_frame_highlight2")
	end
	
	screen:activate_widget(string.format("icon_%s_highlight", new_location))
	gp_mode.index = new_location
	if(gp_navigation[gp_mode.index].widget) then
		local widget = GetGamepadWidget(gp_mode.index)
		MenuSystem.SetGamepadWidget(screen, widget)
		if gp_navigation[gp_mode.index].pad then
			screen:OnMouseEnter(screen:get_widget_id(widget), self:GetX() + screen:get_widget_x(widget), self:GetY() + screen:get_widget_y(widget))
		end
		--screen:OnButtonAvailable(screen:get_widget_id(widget), screen:get_widget_x(widget), screen:get_widget_y(widget), true)
	end
end

function BattleGameMenu:OnEventStartTurn(event)
	for i, client in ipairs(self.clients) do
		SCREENS[client]:StartTurn(event)
	end
end

function BattleGameMenu:OnGamepadDPad(user_id, dpad_id, x_value, y_value, add)
	if not add then
		self:OnGamepadDPad(user_id, dpad_id, x_value, y_value, true)
		return Menu.MESSAGE_HANDLED
	end
	
	LOGF("BattleGameMenu:OnGamepadDPad(user_id=%d, dpad_id=%d, x_value=%d, y_value=%d", user_id, dpad_id, x_value, y_value)
	
	if gp_mode.location == MODE_GRID then
		if (self.selected or dpad_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_JOYSTICK_2)) and
			(not self:GetWorld():IsInTargettingMode()) then
			if IsOpen("OppTutorialMenu") then
				SCREENS.OppTutorialMenu:Close()
			end
			self.selected = false
			self:GetWorld():DoGamepadSwap(x_value, y_value)
		else
			self:GetWorld():ShowGamepadSelection(x_value, y_value)
		end
	else
		LuaGamepad.NavigateUI(self, x_value, y_value, gp_navigation, gp_mode.index)
	end
	
	return Menu.MESSAGE_HANDLED
end

--These functions need to be removed after the OnGamepadJoystick bug is fixed
function BattleGameMenu:OnGamepadJoystick(user, joystick, x, y)
	if _G.Joystick then
		_G.Joystick.OnGamepadJoystick(user, joystick, x, y)
	end
	
	return Menu.MESSAGE_HANDLED
end

function BattleGameMenu:OnTimer(time)
	if _G.Joystick then
		_G.Joystick.OnTimer(self, time)
	end
	self:OnTimerPlatform(time)
	return Menu.MESSAGE_HANDLED
end

function BattleGameMenu:IsPopupsEnabled()
	if (not IsGamepadActive()) or (gp_mode.show_info) then
		return true
	end
	return false
end

function BattleGameMenu:ShowStealthHalo()
	if not self.stealth_count then
		self.stealth_count = 0
	end
	
	self.stealth_count = self.stealth_count + 1
	
	if not self.stealth_thread then
		local MenuHelpers = require("Utils.MenuHelpers")
		self.stealth_thread = scheduler:AddChildThread(self, function()
			while self:IsOpen() do -- continue this until the thread is killed
				MenuHelpers.FadeUpWidget(self.stealth_thread, "EffectsMenu", "icon_fullscreen_halo", 0.2, 0.5, 0.12)
				scheduler:Sleep(2500)
				MenuHelpers.FadeDownWidget(self.stealth_thread, "EffectsMenu", "icon_fullscreen_halo", 0.5, 0.2, 0.12)
				scheduler:Sleep(2500)
			end
			SCREENS.EffectsMenu:set_alpha("icon_fullscreen_halo", 0.0)
		end)
	end
end

function BattleGameMenu:HideStealthHalo()
	if self.stealth_count then
		self.stealth_count = self.stealth_count - 1
	else
		self.stealth_count = 0
	end

	if self.stealth_thread and (self.stealth_count == 0) then
		scheduler:StopThread(self.stealth_thread)
		scheduler:StopAllThreadsForObject(self)
		self.stealth_thread = nil
	end
	SCREENS.EffectsMenu:HideStealthHalo()
end

-- DO NOT RETURN ANYTHING FROM THIS FILE, IF THIS CONFUSES YOU, COME
-- AND TALK TO NICK!!

