use_safeglobals()

class "CombatResultsMenu" (Menu);

local Listbox = require("Utils/ListboxHelpers")

function CombatResultsMenu:__init()
	super()
	
	self:Initialize("Assets\\Screens\\CombatResultsMenu.xml")
end

function CombatResultsMenu:OnOpen()
	self.turn_lb = {}
	self.defeated_lb = {}
	
	if _G.IsDemo() then
		Achievements.DisplayNewDemoAchievements()
	end
	
	return Menu.OnOpen(self)
end

function CombatResultsMenu:OnAnimOpen(data)
	if (data == 1) then
		SCREENS.BattleGameMenu:Close()
	end
end

function CombatResultsMenu:OnRefresh()
	self:hide_widget("grp_loading")
	
	if IsGamepadActive() then
		self:activate_widget("grp_gamepad")
		self:hide_widget("grp_buttons")
	else
		self:hide_widget("grp_gamepad")
		self:activate_widget("grp_buttons")
	end
	
	if self.victory then
		self:set_text("str_heading", "[WON_HEAD]")
		self:set_image("icon_backdrop1", "img_backdrop_victory1")
		self:set_image("icon_backdrop2", "img_backdrop_victory2")
		self:set_image("icon_backdrop3", "img_backdrop_victory3")
		self:set_image("icon_backdrop4", "img_backdrop_victory4")
	else
		self:set_text("str_heading", "[LOST_HEAD]")
		self:set_image("icon_backdrop1", "img_backdrop_defeat1")
		self:set_image("icon_backdrop2", "img_backdrop_defeat2")
		self:set_image("icon_backdrop3", "img_backdrop_defeat3")
		self:set_image("icon_backdrop4", "img_backdrop_defeat4")
	end
	
	if self.xp then
		self:set_text_raw("str_xp",   tostring(self.xp))
	end
	
	if self.gold then
		self:set_text_raw("str_gold", tostring(self.gold))
	end
	
	if Listbox.ListExists("combatresults") then
		Listbox.UpdateList("combatresults")
	end
end

function CombatResultsMenu:Setup(session, callback)
	assert(session)
	assert(callback)
	
	table.log(session)
	
	self.hero = session.heroes[1][1]
	self.enemy = session.heroes[2][1]
	self.items = {}
	self.victory = session.results.winningTeam == 1
	self.closeCallback = callback
	self.numItems = 0
	self.currentMessage = 1
	self.messages = {}
	self.check_lb = false

	local enemy_max_hp = self.enemy:GetMaxHealth()
	local monster_name = translate_text(self.enemy:GetAttribute("name"))
	
	self:OnLineCheck()
	
	if self.victory then
		self:set_text("str_heading", "[WON_HEAD]")
		self:set_image("icon_backdrop1", "img_backdrop_victory1")
		self:set_image("icon_backdrop2", "img_backdrop_victory2")
		self:set_image("icon_backdrop3", "img_backdrop_victory3")
		self:set_image("icon_backdrop4", "img_backdrop_victory4")
		self.gold = self.enemy:GetEnemyGold()
		self.xp   = math.floor(enemy_max_hp * self.enemy.def.experiencePerHP)
		
		if session.isAmbush then
			self.xp = 2 * self.xp
		end
		
		-- fill the box with the enemies trade items
		local commodities ={"wood", "metal", "leather", "amber", "pearl", "emerald", "ruby", "sapphire", "fang", "crystal"}
		local commodity, amount
		
		for i=1, #commodities do
			commodity = commodities[i]
			amount = self.enemy:GetCommodityAmount(commodity)
			if amount > 0 then
				self:AddItem(commodity, amount)
				self.enemy:RemoveCommodity(commodity, amount)
			end
		end
		
		self:DisplayResults(session)
		self:MonsterWinCheck(monster_name, session.results.turnCount)
		
	else
		self:set_text("str_heading", "[LOST_HEAD]")
		self:set_image("icon_backdrop1", "img_backdrop_defeat1")
		self:set_image("icon_backdrop2", "img_backdrop_defeat2")
		self:set_image("icon_backdrop3", "img_backdrop_defeat3")
		self:set_image("icon_backdrop4", "img_backdrop_defeat4")
		
		self.gold = 0
		self.xp   = math.floor((enemy_max_hp - session.results.health2) * self.enemy.def.experiencePerHP / 2)
		
		self:DisplayResults(session)
		self:MonsterLoseCheck(monster_name)
	end
end

function CombatResultsMenu:DisplayResults(session)

	self:OnLineCheckWidgets()

	local levelDiff = self.enemy:GetAttribute("level") - self.hero:GetAttribute("level")
	
	self.xp = math.floor(math.max(self.xp * (1+(0.1 * levelDiff)), 0)) -- apply bonus xp for level difference
	self.xp = self.xp + session.results.end_xp1
	
	self:set_text_raw("str_gold", tostring(self.gold))
	self:set_text_raw("str_xp",   tostring(self.xp))
	self:SetupListbox()
	Listbox.UpdateList("combatresults")
end

function CombatResultsMenu:AddItem(name, amount)
	LOGF("AddItem(%s, %s)", tostring(name), tostring(amount))
	
	self.numItems = self.numItems + 1
	
	if self.numItems <= 5 then
		table.insert(self.items, {
			name = name,
			amount = amount,
		})
	else
		error("There are more items than slots, this should not be possible")
	end
end

function CombatResultsMenu:GiveRewards()
	if self.victory then
		for i=1, self.numItems do
			self.hero:AddCommodity(self.items[i].name, self.items[i].amount)
		end
	end
	
	if self.hero then
		self.hero:AwardExperience(self.xp)
		self.hero:AwardGold(self.gold)
	end
end

function CombatResultsMenu:OnFinishConfirm()
	self:CommitResults()
	self:GiveRewards()
	self.closeCallback(self.victory)
	self.closeCallback = nil
	self:Close()
end

function CombatResultsMenu:OnButton(button_id, x_value, y_value)
	if button_id == self:get_widget_id("butt_ok") then
		self:OnFinishConfirm()
	else
		return Menu.MESSAGE_NOT_HANDLED
	end
	
	return Menu.MESSAGE_HANDLED
end

function CombatResultsMenu:OnGamepadButton(user_id, button_id, up)
	if up == 0 then
		return Menu.MESSAGE_HANDLED
	end
	
	if button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_OKAY) then
		self:OnFinishConfirm()
	end
	
	return Menu.MESSAGE_HANDLED
end

function CombatResultsMenu:ShowItemPopup(slot, x, y, id)
	assert(slot)
	assert(x)
	assert(y)
	
	local item = self.items[slot]
	
	if not item then
		return
	end
end

function CombatResultsMenu:OnMouseEnter(pad_id, x_value, y_value)
	-- We should handle this
	return Menu.MESSAGE_NOT_HANDLED
end

function CombatResultsMenu:OnMouseLeave(id, x, y)
	LOGF("CombatResultsMenu:OnMouseLeave(%d, x=%d, y=%d)", id, x, y)
	
	if id == self:get_widget_id("pad_item1") then
		close_custompopup_menu()
	elseif id == self:get_widget_id("pad_item2") then
		close_custompopup_menu()
	elseif id == self:get_widget_id("pad_item3") then
		close_custompopup_menu()
	elseif id == self:get_widget_id("pad_item4") then
		close_custompopup_menu()
	elseif id == self:get_widget_id("pad_item5") then
		close_custompopup_menu()
	else
		return Menu.OnMouseLeave(self, id, x, y)
	end
	
	return Menu.MESSAGE_HANDLED
end

function CombatResultsMenu:SetupListbox()
	Listbox.AddListbox("combatresults", 5, self.items)
	Listbox.SetCallback("combatresults", "SetData",
	function(entry, field)
		LOGF("[CombatResultsMenu:SetupListbox] SetData: set data: %s(%s), %s", tostring(entry), tostring(entry.name), tostring(field))
		LOGF("[CombatResultsMenu:SetupListbox] SetData: setting image to '%s'", string.format("img_tradeitem_%s", entry.name))
		
		self:activate_widget(string.format("icon_item%d", field))
		self:activate_widget(string.format("str_item%d_name", field))
		self:activate_widget(string.format("str_item%d_value", field))

		self:set_image(string.format("icon_item%d", field), string.format("img_tradeitem_%s", entry.name))
		self:set_text(string.format("str_item%d_name", field), string.format("[%s_NAME]", string.upper(entry.name)))
		local hero_stock = self.hero:GetCommodityAmount(entry.name)
		self:set_text(string.format("str_item%d_value", field), string.format("(%d)+%d", hero_stock, entry.amount))
	end)
	Listbox.SetCallback("combatresults", "SelectEntry", function(entry, field) end)
	Listbox.SetCallback("combatresults", "DeselectEntry", function(entry, field) end)
	Listbox.SetCallback("combatresults", "NoData",
	function(field)
		self:hide_widget(string.format("icon_item%d", field))
		self:hide_widget(string.format("icon_item%d_overlay", field))
		self:hide_widget(string.format("str_item%d_value", field))
		self:hide_widget(string.format("str_item%d_name", field))
	end)
end

function CombatResultsMenu:OnClose()
	self.messages = nil
	self.currentMessage = nil
	self.lbmode = nil
	self.okay_pressed = nil
	
	self.hero = nil
	self.enemy = nil
	self.closeCallback = nil
	
	Listbox.ClearListbox("combatresults")
	
	-- Update Music
	if rawget(SCREENS, "WorldMapMenu") and SCREENS.WorldMapMenu:IsOpen() then
		MusicManager:SetState("WORLD_BEGIN")
	else
		MusicManager:SetState("MENU")
	end
	
	return Menu.OnClose(self)
end

-- Any platform specific stuff in here
dofile("Assets/Scripts/Screens/CombatResultsMenuPlatform.lua")

return ExportSingleInstance("CombatResultsMenu")