use_safeglobals()

local MAX_ENTRIES    = 5
local MAX_SPELLS     = 5
local STAT_BONUS     = 1
local MAX_MANA_BONUS = 5
local DAMAGE_BONUS   = 1

local SELECTION_OFFSET_X = 308
local SELECTION_OFFSET_Y = 326
local SELECTION_OFFSET_HEIGHT = 68

local HeroStatsConstants = require( "Hero/StatsConstants" )

class "LevelUpMenu" (Menu);

function LevelUpMenu:__init()
	super()
	self:Initialize("Assets\\Screens\\LevelUpMenu.xml")
end

function LevelUpMenu:OnOpen()
	SoundSystem.Play("pq2audio/ui/levelup") 
	
	if _G.Joystick then
		_G.Joystick.InitJoystick()
	end
	
	--SHOW TUTORIAL ONCE
	local hero = HeroManager:GetActiveHero()
	if not hero:GetAttribute("levelUpTutorialShown") then
		open_conversation_menu(
			"Assets/Conversations/LEVEL_UP_TUTORIAL.xml",
			hero:GetName(), 
			hero:GetConversationPortrait(), 
			hero:GetGenderForConversation(), 
			"","","", 
			function ()-- conversation finished, start game
				hero:SetAttribute("levelUpTutorialShown",true)
			end, 
			x or GetScreenWidth()*0.5,
			y or GetScreenHeight()*0.5
		)
	end
	
	return Menu.OnOpen(self)
end

function LevelUpMenu:OnRefresh()
	if IsGamepadActive() then
		self:activate_widget("str_gp_a")
		self:activate_widget("str_gp_y")
		self:activate_widget("icon_gp_a")
		self:activate_widget("icon_gp_y")
		self:hide_widget("butt_done")
		
		self.selectedItem = 1
		self:set_text("str_gp_y", "[MORE_INFO_SHOW]")
	else
		self:hide_widget("str_gp_a")
		self:hide_widget("str_gp_y")
		self:hide_widget("icon_gp_a")
		self:hide_widget("icon_gp_y")
		self:deactivate_widget("butt_done")
	end
	
	self:hide_widget("icon_choice_highlight")
	self:hide_widget("str_spell1")
	self:hide_widget("icon_spell")
	self:hide_widget("icon_spell_bg")
	self:hide_widget("str_spell")
	self:hide_widget("icon_spell_text_fill")
	
	for i=0,5 do
		self:set_image(string.format("icon_choice%d_bg", i), "img_LargeButton_409x64")
	end
	
	if self.selectedItem then
		self:HighlightSelection()
		if not IsGamepadActive() then
			self:activate_widget("butt_done")
		end
	end
	
	if self.hero and self.currentLevel then
		self:SetLevelGainData(self.hero, self.currentLevel)
	end
end

function LevelUpMenu:OnClose()
	close_custompopup_menu()
	
	if IsOpen("WorldMapMenu") then
		SCREENS.WorldMapMenu.hero:EnableMovement()
	end
	
	if self.completedCallback then
		self.completedCallback()
		self.completedCallback = nil
	end
	
	self.hero = nil
	self.showInfo = nil
	self.selectedItem = nil
	
	return Menu.OnClose(self)
end

function LevelUpMenu:SetCompletedCallback(cb)
	self.completedCallback = cb
end

function LevelUpMenu:OnMouseLeftButton(pad_id, x_value, y_value)
	if pad_id == self:get_widget_id("pad_spell") then
		SCREENS.SpellbookMenu:SetHero(self.hero)
		SCREENS.SpellbookMenu:Open()
		
		return Menu.MESSAGE_HANDLED
	end
	
	for i=1,5 do
		if pad_id == self:get_widget_id(string.format("pad_choice%d", i)) then
			SoundSystem.Play("pq2audio/ui/itemhover")
			self.selectedItem = i
			self:HighlightSelection()
			if not IsGamepadActive() then
				self:activate_widget("butt_done")
			end
			
			return Menu.MESSAGE_HANDLED
		end
	end
	
	return Menu.MESSAGE_HANDLED
end

function LevelUpMenu:OnMouseEnter(pad_id, x_value, y_value)
	x_value = self:get_widget_abs_x("pad_spell") + (self:get_widget_w("pad_spell")/2) + (self:get_widget_w("icon_spell")/2)   --x_value + self:get_widget_abs_x("pad_spell")
	y_value = self:get_widget_abs_y("pad_spell") + (self:get_widget_h("pad_spell"))

	if pad_id == self:get_widget_id("pad_spell") then
		if self.bonusSpell then
			local fmt, data = _G.GetSpellToolTipFormatAndData(self.bonusSpell)
			
			if fmt and data then
				open_custompopup_menu(fmt, data, x_value, y_value, 2, 256, "LevelUpMenu", 0)
			end
			
			SoundSystem.Play("pq2audio/ui/itemhover")
			self:set_image("icon_spell_bg", "img_LargeButton_409x64_hi")
		end
	else
		for i=1,5 do
			if pad_id == self:get_widget_id(string.format("pad_choice%d", i)) then
				local stat = STATS_ORDER[i]
				local showSpecialStatEffect = false
				if stat == "intelligence" then
					showSpecialStatEffect = true
				end
				
				local fmt, data = self:GetAttributeToolTipFormatAndData(stat, showSpecialStatEffect)
				
				if fmt and data then
					open_custompopup_menu(fmt, data, x_value, y_value, 2, 256, "LevelUpMenu", i)
				end
				
				if self.selectedItem ~= i then
					SoundSystem.Play("pq2audio/ui/itemhover")
					self:set_image(string.format("icon_choice%d_bg", i), "img_LargeButton_409x64_hi")
				end
				
				return Menu.MESSAGE_HANDLED
			end
		end
	end
	
	return Menu.MESSAGE_HANDLED
end

function LevelUpMenu:OnMouseLeave(pad_id, x_value, y_value)
	if pad_id == self:get_widget_id("pad_spell") then
		close_custompopup_menu(0)
		
		self:set_image("icon_spell_bg", "img_LargeButton_409x64")
	else
		for i=1,5 do
			if pad_id == self:get_widget_id(string.format("pad_choice%d", i)) then
				close_custompopup_menu(i)
				
				if self.selectedItem ~= i then
					self:set_image(string.format("icon_choice%d_bg", i), "img_LargeButton_409x64")
				end
				
				return Menu.MESSAGE_HANDLED
			end
		end
	end
	
	return Menu.MESSAGE_HANDLED
end

function LevelUpMenu:AwardSelectedBonus()
	assert(self.selectedItem, "Nothing selected")

	-- award bonus attribute
	self.hero:SetAttribute(STATS_ORDER[self.selectedItem], self.hero:GetAttribute(STATS_ORDER[self.selectedItem])+1)
	self.hero:CalculateStats()
end

function LevelUpMenu:SetLevelGainData(hero, curr_level)
	self.currentLevel = curr_level
	
	self.hero = hero
	self.HeroData = require("Hero/" .. tostring(hero:GetAttribute("class")))
	local strHint = ""
	if hero:GetAttribute("class") == "Assassin" then
		strHint = ", Hint: 6 36 36 21 6"
	elseif hero:GetAttribute("class") == "WarMage" then
		strHint = ", Hint: 6 21 36 21 21"
	elseif hero:GetAttribute("class") == "Barbarian" then
		strHint = ", Hint: 28 36 21 10 10"
	elseif hero:GetAttribute("class") == "Inquisitor" then
		strHint = ", Hint: 3 36 28 10 3"
	end
	
	self:set_text_raw("str_level_msg", substitute(translate_text("[LEVELUP_MSG1]"), curr_level, strHint))
	--self:set_text_raw("str_level_value", tostring(curr_level))
	self:set_text_raw("str_hp_msg", substitute(translate_text("[LEVELUP_MSG2]"), self.HeroData.hit_points_per_level))
	--self:set_text_raw("str_hp_value",     tostring(self.HeroData.hit_points_per_level))
	
	local levelBonus = self.HeroData.level_bonuses[curr_level]
	
	self.bonusSpell = levelBonus.spell
	
	if self.bonusSpell then
		-- award bonus spell now so the player can use the spellbook menu
		self.hero:PushAttribute("knownSpells", self.bonusSpell)
		if self.hero:NumAttributes("activeSpells") < MAX_SPELLS then
			self.hero:PushAttribute("activeSpells", self.bonusSpell)
		end
		
		local info = SPELLS[self.bonusSpell]
		
		self:activate_widget("str_spell1")
		self:activate_widget("icon_spell")
		self:activate_widget("icon_spell_bg")
		self:set_image("icon_spell", info.icon)
		self:set_text("str_spell1", string.format("[%s_NAME]", info.id))
	end
	
	for i=1,MAX_ENTRIES do
		self:activate_widget(string.format("icon_choice%d", i))
		self:activate_widget(string.format("str_choice%d",  i))
		self:activate_widget(string.format("pad_choice%d", i))
		self:activate_widget(string.format("icon_choice%d_bg", i))
	end
	
	self:set_image("icon_choice1", "img_levelup_strength")
	self:set_image("icon_choice2", "img_levelup_agility")
	self:set_image("icon_choice3", "img_levelup_intelligence")
	self:set_image("icon_choice4", "img_levelup_stamina")
	self:set_image("icon_choice5", "img_levelup_morale")
	
	--self:set_text_raw("str_choice1", string.format("+%d %s", STAT_BONUS, translate_text("[STRENGTH]")))
	--self:set_text_raw("str_choice2", string.format("+%d %s", STAT_BONUS, translate_text("[AGILITY]")))
	--self:set_text_raw("str_choice3", string.format("+%d %s", STAT_BONUS, translate_text("[INTELLIGENCE]")))
	--self:set_text_raw("str_choice4", string.format("+%d %s", STAT_BONUS, translate_text("[STAMINA]")))
	--self:set_text_raw("str_choice5", string.format("+%d %s", STAT_BONUS, translate_text("[MORALE]")))
	self:set_text_raw("str_choice1", string.format("+%d %s (%d)", STAT_BONUS, translate_text("[STRENGTH]"), self.hero:GetStat("strength")))
	self:set_text_raw("str_choice2", string.format("+%d %s (%d)", STAT_BONUS, translate_text("[AGILITY]"), self.hero:GetStat("agility")))
	self:set_text_raw("str_choice3", string.format("+%d %s (%d)", STAT_BONUS, translate_text("[INTELLIGENCE]"), self.hero:GetStat("intelligence")))
	self:set_text_raw("str_choice4", string.format("+%d %s (%d)", STAT_BONUS, translate_text("[STAMINA]"), self.hero:GetStat("stamina")))
	self:set_text_raw("str_choice5", string.format("+%d %s (%d)", STAT_BONUS, translate_text("[MORALE]"), self.hero:GetStat("morale")))
	
	self:activate_widget("str_rewards")
	
	if IsGamepadActive() then
		SoundSystem.Play("pq2audio/ui/itemhover")
		self.selectedItem = 1
		self:HighlightSelection()
	end
	
	if curr_level == 21 then
		--SHOW TUTORIAL ONCE
		if not self.hero:GetRebuildUnlockTutorialSeen() then
			open_conversation_menu(
				"Assets/Conversations/REBUILDING_UNLOCK_TUTORIAL.xml",
				self.hero:GetName(), 
				self.hero:GetConversationPortrait(), 
				self.hero:GetGenderForConversation(), 
				"","","", 
				function ()-- conversation finished, start game
					self.hero:SetRebuildUnlockTutorialSeen()
				end, 
				x or GetScreenWidth()*0.5,
				y or GetScreenHeight()*0.5
			)
		end
	end
end

function LevelUpMenu:OnGamepadJoystick(user_id, joystick_id, x_value, y_value)
	if _G.Joystick then
		_G.Joystick.OnGamepadJoystick(user_id, joystick_id, x_value, y_value)
	end
	return Menu.MESSAGE_HANDLED
end

function LevelUpMenu:OnGamepadDPad(user_id, pad_id, x_value, y_value)
	if x_value == 0 and y_value == 0 then
		return Menu.MESSAGE_NOT_HANDLED
	end
	
	local nextSelectedItem
	
	if y_value > 0 then--dpad up
		if self.selectedItem > 1 then
			nextSelectedItem = self.selectedItem - 1
		elseif self.selectedItem == 1 and self.bonusSpell then
			nextSelectedItem = 0
		else --End of list - cycle to bottom				
			nextSelectedItem = 5
		end
	else--dpad down
		if self.selectedItem < 5 then
			nextSelectedItem = self.selectedItem + 1
		elseif self.selectedItem == 5 then--End of list - cycle to top
			if self.bonusSpell then
				nextSelectedItem = 0
			else
				nextSelectedItem = 1
			end
		end
	end
	if nextSelectedItem == 0 then
		--Hide (A) Done icon
		self:hide_widget("str_gp_a")
		self:hide_widget("icon_gp_a")
	else
		--Show 
		self:activate_widget("str_gp_a")
		self:activate_widget("icon_gp_a")
	end
	
	self.selectedItem = nextSelectedItem

	self:HighlightSelection()
	self:ShowInfo()
	
	SoundSystem.Play("pq2audio/ui/itemhover")
	
	if not IsGamepadActive() then
		self:activate_widget("butt_done")
	end
	
	return Menu.MESSAGE_HANDLED
end

function LevelUpMenu:OnGamepadButton(user_id, button_id, up)
	if up == 0 then
		return Menu.MESSAGE_HANDLED
	end
	
	if button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_OKAY) then
		if self.selectedItem > 0 then
			SoundSystem.Play("pq2audio/ui/menu_select")
			self:AwardSelectedBonus()
			self:Close()
			
			return Menu.MESSAGE_HANDLED
		else
			SCREENS.SpellbookMenu:SetHero(self.hero)
			SCREENS.SpellbookMenu:Open()
			
			return Menu.MESSAGE_HANDLED
		end
	end
	
	--Toggle showInfo
	if button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_TOP) then
		self.showInfo = not self.showInfo
		
		if self.showInfo then
			self:set_text("str_gp_y", "[MORE_INFO_HIDE]")
		else
			self:set_text("str_gp_y", "[MORE_INFO_SHOW]")
		end
		
		self:ShowInfo()
	end

	return Menu.MESSAGE_NOT_HANDLED
end

function LevelUpMenu:OnKey(key)
	return Menu.MESSAGE_NOT_HANDLED
end

function LevelUpMenu:OnButton(button_id, x_value, y_value)
	if button_id == self:get_widget_id("butt_done") then
		SoundSystem.Play("pq2audio/ui/menu_select")
		self:AwardSelectedBonus()
		self:Close()
	end
	return Menu.MESSAGE_HANDLED
end

function LevelUpMenu:OnTimer(time)
	if _G.Joystick then
		_G.Joystick.OnTimer(self, time)
	end
	return Menu.MESSAGE_HANDLED
end


function LevelUpMenu:HighlightSelection()
	
	-- remove previous highlight
	if self.lastHighlight then
		self:set_image(self.lastHighlight, "img_LargeButton_409x64")		
	end
	
	local widget
	if self.selectedItem > 0 then
		widget = string.format("icon_choice%d_bg", self.selectedItem)
	elseif self.bonusSpell then
		widget = "icon_spell_bg"	
	end
	
	if widget then
		self:set_image(widget, "img_LargeButton_on")
		self.lastHighlight = widget		
	end
end

function LevelUpMenu:ShowInfo()
	if self.lastPopup then
		close_custompopup_menu(self.lastPopup)
	end
	if self.showInfo then
		if self.selectedItem > 0 then
			local stat = STATS_ORDER[self.selectedItem]
			local showSpecialStatEffect = false
			if stat == "intelligence" then
				showSpecialStatEffect = true
			end
			
			local fmt, data = self:GetAttributeToolTipFormatAndData(stat, showSpecialStatEffect)
			
			local widget_name = string.format("pad_choice%d", self.selectedItem)
			LOGF("ShowInfo: %s, %s", tostring(fmt), tostring(data))
			if fmt and data then
				open_custompopup_menu(fmt, data, self:get_widget_w(widget_name)/2, self:get_widget_h(widget_name)/2, 4, 256, "LevelUpMenu", self.selectedItem)
			end
			self.lastPopup = self.selectedItem
		else
			if self.bonusSpell then
				local fmt, data = _G.GetSpellToolTipFormatAndData(self.bonusSpell)
				
				if fmt and data then
					open_custompopup_menu(fmt, data, self:get_widget_w("pad_spell")/2, self:get_widget_h("pad_spell")/2, 2, 256, "LevelUpMenu", 0)
				end
				self.lastPopup = 0
			end
		end
	end

end


function LevelUpMenu:GetAttributeToolTipFormatAndData(attr, addSpecialEffect)
	local fmt = ""
	local data = ""
	
	local WIDTH = 416
	
	local HEADING_Y = 3
	local HEADING_HEIGHT = 40
	
	local SUBTITLE_Y = HEADING_Y + HEADING_HEIGHT
	local SUBTITLE_HEIGHT = 30
	
	local SPECIALEFFECT_Y = SUBTITLE_Y + SUBTITLE_HEIGHT
	local SPECIALEFFECT_HEIGHT = 32
	
	local VALUEEFFECT_Y = SUBTITLE_Y + SUBTITLE_HEIGHT
	local VALUEEFFECT_HEIGHT = 32
	
	if addSpecialEffect then
		VALUEEFFECT_Y = SPECIALEFFECT_Y + SPECIALEFFECT_HEIGHT
	end
	
	local RANKEFFECT_Y = VALUEEFFECT_Y + VALUEEFFECT_HEIGHT
	local RANKEFFECT_HEIGHT = 32
	
	local STARTMANA_Y = RANKEFFECT_Y + RANKEFFECT_HEIGHT
	local STARTMANA_HEIGHT = 32
	
	local EARNEDMANA_Y = STARTMANA_Y + STARTMANA_HEIGHT
	local EARNEDMANA_HEIGHT = 32
	
	local MAXMANA_Y = EARNEDMANA_Y + EARNEDMANA_HEIGHT
	local MAXMANA_HEIGHT = 32	
	
	local PAGE_OFFSET = MAXMANA_Y + MAXMANA_HEIGHT - HEADING_HEIGHT
	
	local EDGE_OFFSET = 3
	local BULLET_OFFSET = 10

	local hero_stats_constants = require( "Hero/StatsConstants" )
	local currentScore = self.hero:GetStat(attr)
	local currentRank = self.hero:GetStatPointsRanking(currentScore).rank
	local nextScore = currentScore + 1
	local nextRank = self.hero:GetStatPointsRanking(nextScore).rank
	
	local str_bullet = "img_levelup_" .. attr
	local upr_attr = string.upper(attr)
	local str_name   = translate_text(string.format("[%s]", upr_attr))
	
	-- Minigame bonus turns
	local str_special_effect
	if addSpecialEffect then
		str_special_effect   = translate_text(string.format("[%s_SPECIALEFFECT]", upr_attr))
	end
	
	local str_value_effect   = translate_text(string.format("[%s_VALUEFFECT]", upr_attr))
	local str_rank_effect    = translate_text(string.format("[%s_RANKEFFECT]", upr_attr))
	local str_mana1_effect   = translate_text(string.format("[%s_STARTMANAEFFECT]", upr_attr))
	local str_mana2_effect   = translate_text(string.format("[%s_EARNEDMANAEFFECT]", upr_attr))
	local str_mana3_effect   = translate_text(string.format("[%s_MAXMANAEFFECT]", upr_attr))
	
	local current_stat_value_effect = currentScore
	local current_stat_rank_effect = currentRank
	local current_stat_mana_start_effect = math.floor( current_stat_rank_effect * hero_stats_constants.mana_start_multiplier )
	local current_stat_mana_earn_effect = math.floor( current_stat_rank_effect * hero_stats_constants.mana_earn_multiplier )
	local current_stat_mana_max_effect = math.floor( current_stat_rank_effect * hero_stats_constants.mana_max_multiplier )
		
	local next_stat_value_effect = nextScore
	local next_stat_rank_effect = nextRank
	local next_stat_mana_start_effect = math.floor( next_stat_rank_effect * hero_stats_constants.mana_start_multiplier )
	local next_stat_mana_earn_effect = math.floor( next_stat_rank_effect * hero_stats_constants.mana_earn_multiplier )
	local next_stat_mana_max_effect = math.floor( next_stat_rank_effect * hero_stats_constants.mana_max_multiplier )
	
	
	-- Title is the attribute name
	fmt = fmt .. "//SHADING/img_TextBackground_Red"
	fmt, data = TooltipHelpers.AddTitle(fmt, data, HEADING_Y+EDGE_OFFSET, "font_uitext", str_name)
	fmt = fmt .. "//SHADING/off"
	
	-- Show the current levels amounts
	fmt = fmt .. "//SHADING/img_TextBackground_Purple"
	fmt, data = TooltipHelpers.AddTitle(fmt, data, SUBTITLE_Y, "font_uitext", substitute(translate_text("[LEVELUP_CURRENT]"), tostring(currentScore)))
	fmt = fmt .. "//SHADING/off"
	
	if addSpecialEffect then
		-- spell penetration
		fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, SPECIALEFFECT_Y+4, 24, 24, str_bullet)
		fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, SPECIALEFFECT_Y, WIDTH-32, SPECIALEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_special_effect, current_stat_value_effect))
	end
	
	-- N% chance for weapon critical
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, VALUEEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, VALUEEFFECT_Y, WIDTH-32, VALUEEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_value_effect, current_stat_value_effect))
	
	-- +N Damage from Skull Gems
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, RANKEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, RANKEFFECT_Y, WIDTH-32, RANKEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_rank_effect, current_stat_rank_effect))
	
	-- +N Start Red mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, STARTMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, STARTMANA_Y, 238, STARTMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana1_effect, current_stat_mana_start_effect))
	
	-- +N Earned Red Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, EARNEDMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, EARNEDMANA_Y, WIDTH-32, EARNEDMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana2_effect, current_stat_mana_earn_effect))
	
	-- +N Max Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, MAXMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, MAXMANA_Y, WIDTH-32, MAXMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana3_effect, current_stat_mana_max_effect))
	
	
	
	-- Show the NEXT levels amounts
	fmt = fmt .. "//SHADING/img_TextBackground_Purple"
	fmt, data = TooltipHelpers.AddTitle(fmt, data, PAGE_OFFSET+SUBTITLE_Y, "font_uitext", substitute(translate_text("[LEVELUP_NEXT]"), tostring(nextScore)))
	fmt = fmt .. "//SHADING/off"
	
	if addSpecialEffect then
		-- spell penetration
		fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+SPECIALEFFECT_Y+4, 24, 24, str_bullet)
		fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+SPECIALEFFECT_Y, WIDTH-32, SPECIALEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_special_effect, next_stat_value_effect))
	end
	
	-- N% chance for weapon critical
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+VALUEEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+VALUEEFFECT_Y, WIDTH-32, VALUEEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_value_effect, next_stat_value_effect))
	
	-- +N Damage from Skull Gems
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+RANKEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+RANKEFFECT_Y, WIDTH-32, RANKEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_rank_effect, next_stat_rank_effect))
	
	-- +N Start mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+STARTMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+STARTMANA_Y, 238, STARTMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana1_effect, next_stat_mana_start_effect))
	
	-- +N Earned Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+EARNEDMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+EARNEDMANA_Y, WIDTH-32, EARNEDMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana2_effect, next_stat_mana_earn_effect))
	
	-- +N Max Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+MAXMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+MAXMANA_Y, WIDTH-32, MAXMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana3_effect, next_stat_mana_max_effect))
	
	
	
	return fmt .. "//", data .. "//"
end

return ExportSingleInstance("LevelUpMenu")
