use_safeglobals()

class "PreBattleMenu" (Menu)

function PreBattleMenu:__init()
	super()
	
	self:Initialize("Assets\\Screens\\PreBattleMenu.xml")
end

function PreBattleMenu:OnOpen()
	self:EnableInput()
	
	-- Update Music
	if self.monsterHero.def.boss then
		MusicManager:SetParam("boss_encounter", true)
	else
		MusicManager:SetParam("boss_encounter", false)
	end
	MusicManager:SetState("PREBATTLE")
	
	return Menu.OnOpen(self)
end

function PreBattleMenu:OnAnimOpen(data)
	if (data == 1) then
		HideBackdropMenu()
		local def = self.monsterHero.def
		if def and def.voice then
			SoundSystem.Play(string.format("pq2audio/vox/%s_agro", def.voice))
		end
	end
end

function PreBattleMenu:OnRefresh()
	if IsGamepadActive() then
		self:activate_widget("grp_xboxbuttons")
		self:hide_widget("grp_pcbuttons")
	else
		self:hide_widget("grp_xboxbuttons")
		self:activate_widget("grp_pcbuttons")
	end
	
	local backdrop = LEVELS[self.playerHero:GetCurrentLocationLevelName()].combatBanner
	for i=0,1 do
		for j=0,1 do
			self:set_image(string.format("icon_level_backdrop%d%d", i, j), string.format("%s%d%d", backdrop, i, j))
		end
	end
	
	self:set_image("icon_portrait_player", self.playerHero:GetBigPortrait())
	self:set_image("icon_portrait_enemy", self.monsterHero:GetBigPortrait())
	
	self:set_text_raw("str_name_value1", self.playerHero:GetName())
	self:set_text_raw("str_name_value2", self.monsterHero:GetName())
	self:set_text_raw("str_name_value3", self.monsterHero:GetName())
	self:set_text("str_description", self.monsterHero:GetDescriptionTag())
	
	local def = self.monsterHero.def
	local hero = HeroManager:GetActiveHero()
	
	local giveGold = ""
	if def.gold then
		giveGold = "gold, "
	end
	
	local gemDrop = ""
	if def.catalystItem then
		gemDrop = string.format("%s+", string.upper(def.catalystItem))
	end
	
	local expGainAvg = self.monsterHero:GetMaxHealth() * self.monsterHero.def.experiencePerHP / 1000
	local expGainStr = string.format("exp %.1fk", expGainAvg)
	
	local quickNum = ""
	local listMonsterSize = 0
	if self.selectableMonster then
		self.monsterList = GetListOfMonsters(self.playerHero, true)
		self:DetermineMonsterIndex()
		
		for index, mon in pairs(self.monsterList) do
			listMonsterSize = listMonsterSize + 1
		end
		quickNum = string.format("#%d/%d  ", self.monsterIndex, listMonsterSize)
		
		self:activate_widget("grp_monsterselect")
	else
		self:hide_widget("grp_monsterselect")
	end
	
	self:set_text("str_infoplus", string.format("%sÆ: %s%s%s", quickNum, gemDrop, giveGold, expGainStr))
end

function PreBattleMenu:OnClose()
	if self.closeReason == "flee" then
		if IsOpen("WorldMapMenu") then
			MusicManager:SetState("WORLD_BEGIN")
		else
			MusicManager:SetState("MENU")
		end
	end
	
	-- clear some members otherwise they will hang around till the next battle
	-- (not good style)
	self.arguments = nil
	self.transitionFrom = nil
	self.playerHero = nil
	self.monsterHero = nil
	self.onResult = nil
	self.onFinished = nil
	self.onFlee = nil
	self.onFight = nil
	self.params = nil
	self.selectableMonster = nil
	self.monsterList = nil
	self.lockInput = nil
	
	return Menu.OnClose(self)
end

function PreBattleMenu:SetupAndOpen(arguments)

	assert(arguments)
	assert(arguments.transitionFrom)
	assert(arguments.playerHero)
	assert(arguments.monsterHero)
	
	-- other parameters in arguments (all optional)
	--   arguments.onResult
	--   arguments.onFinished
	--   arguments.onFight
	--   arguments.onFlee
	--   arguments.params
	
	SoundSystem.Play("pq2audio/ui/battle")
	
	self.arguments = arguments
	self.transitionFrom = arguments.transitionFrom
	self.playerHero = arguments.playerHero
	self.monsterHero = arguments.monsterHero
	self.onResult = arguments.onResult
	self.onFinished = arguments.onFinished
	self.onFlee = arguments.onFlee
	self.onFight = arguments.onFight
	self.params = arguments.params
	self.selectableMonster = arguments.selectableMonster
	
	-- Sanity catch all to ensure that if hero stats are somehow bugged, they are refreshed just prior to a battle.
	self.playerHero:InitializeStats()
	self.playerHero:CalculateStats()
	
	self:Open()
end

function PreBattleMenu:DisableInput()
	self.lockInput = true
	self:deactivate_widget("butt_prev")
	self:deactivate_widget("butt_next")
	if not IsGamepadActive() then
		self:deactivate_widget("grp_pcbuttons") --fight/flee buttons aren't used on XBLA so they'll be hidden
	end
end

function PreBattleMenu:EnableInput()
	self.lockInput = nil
	self:activate_widget("butt_prev")
	self:activate_widget("butt_next")
	if not IsGamepadActive() then
		self:activate_widget("grp_pcbuttons") --fight/flee buttons aren't used on XBLA
	end
end

function PreBattleMenu:OnButton(buttonId, clickX, clickY)
	if self.lockInput then
		return Menu.MESSAGE_HANDLED
	end

	if buttonId == self:get_widget_id("butt_fight") then
		self:DisableInput()
		SCREENS[self.transitionFrom]:Close()
		SoundSystem.Play("pq2audio/ui/battleload")
		self:StartAnimation("fight")
	elseif buttonId == self:get_widget_id("butt_flee") then
		self:DisableInput()
		self:StartAnimation("flee")
		SoundSystem.Play("pq2audio/ui/battleclose")
	elseif self.selectableMonster and buttonId == self:get_widget_id("butt_prev") then
		self:DisableInput()
		self:PreviousMonster()
	elseif self.selectableMonster and buttonId == self:get_widget_id("butt_next") then
		self:DisableInput()
		self:NextMonster()
	else
		--return Menu.OnButton(self, buttonId, clickX, clickY)
		return Menu.MESSAGE_NOT_HANDLED
	end
	
	return Menu.MESSAGE_HANDLED
end

function PreBattleMenu:OnGamepadButton(user_id, button_id, up)
	if up == 0 or self.lockInput then
		return Menu.MESSAGE_HANDLED
	end
	
	if button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_OKAY) then
		SCREENS[self.transitionFrom]:Close()
		self:DisableInput()
		SoundSystem.Play("pq2audio/ui/battleload")
		self:StartAnimation("fight")
		return Menu.MESSAGE_HANDLED	
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CANCEL) then
		self:DisableInput()
		ShowBackdropMenu()
		self:StartAnimation("flee")
		SoundSystem.Play("pq2audio/ui/battleclose")
		return Menu.MESSAGE_HANDLED
	end

	return Menu.MESSAGE_NOT_HANDLED
end

function PreBattleMenu:OnAnimFight(data)
	if data == 1 then
		self.closeReason = "fight"
		
		if self.onFight then
			self.onFight(self.arguments)
		end
		
		StartBattle(self.transitionFrom, self.playerHero, self.monsterHero, self.onResult, self.onFinished, self.onFlee, self.params)
		
		self:Close()
	end
	
	self:activate_widget("butt_fight")
	self:activate_widget("butt_flee")
	
	return Menu.MESSAGE_HANDLED
end

function PreBattleMenu:OnAnimFlee(data)
	if data == 1 then
		self.closeReason = "flee"
		self:Flee()
	end
	
	self:activate_widget("butt_fight")
	self:activate_widget("butt_flee")
	
	return Menu.MESSAGE_HANDLED
end

-- note: I have moved the call to close to back after the flee code
-- so that self.onFlee will still actually exist to use :p
-- if this causes any problems, make a local variable of self.onFlee
function PreBattleMenu:Flee()
	if self.onFlee then
		self.onFlee()
	end
	self:Close()
end

function PreBattleMenu:PreviousMonster()
	local monsterIndex = self.monsterIndex or self:DetermineMonsterIndex()
	monsterIndex = monsterIndex - 1
	if monsterIndex < 1 then
		self.monsterIndex = #self.monsterList
	else
		self.monsterIndex = monsterIndex
	end
	
	self:StartAnimation("switchmonster_out")
end

function PreBattleMenu:NextMonster()
	local monsterIndex = self.monsterIndex or self:DetermineMonsterIndex()
	monsterIndex = monsterIndex + 1
	if monsterIndex > #self.monsterList then
		self.monsterIndex = 1
	else
		self.monsterIndex = monsterIndex
	end
	
	self:StartAnimation("switchmonster_out")
end

function PreBattleMenu:OnAnimSwitchmonster_out()
	-- self.monsterIndex should have already been changed, switch
	-- does old MonsterHero need to be killed?

	local def = self.monsterHero.def
	if def and def.voice then
			SoundSystem.Unload(string.format("vox_%s_bank", def.voice))
	end

	DestroyHero(self.monsterHero)
	self.monsterHero = CreateMonsterHero(self.monsterList[self.monsterIndex], self.playerHero) 
	self.params.heroes[2][1] = self.monsterHero
	self.arguments.monsterHero = self.monsterHero
	
	def = self.monsterHero.def
	if def and def.voice then
		SoundSystem.Load(string.format("vox_%s_bank", def.voice))
	end
	
	-- set portrait etc
	self:set_image("icon_portrait_enemy", self.monsterHero:GetBigPortrait())
	
	self:set_text_raw("str_name_value2", self.monsterHero:GetName())
	self:set_text_raw("str_name_value3", self.monsterHero:GetName())
	self:set_text("str_description", self.monsterHero:GetDescriptionTag())
	
	local hero = HeroManager:GetActiveHero()
	hero:SetAttribute("dummy", self.monsterIndex)
	
	local giveGold = ""
	if def.gold then
		giveGold = "gold, "
	end
	
	local gemDrop = ""
	if def.catalystItem then
		gemDrop = string.format("%s+", string.upper(def.catalystItem))
	end
	
	local listMonsterSize = 0
	for index, mon in pairs(self.monsterList) do
		listMonsterSize = listMonsterSize + 1
	end
	
	local expGainAvg = self.monsterHero:GetMaxHealth() * self.monsterHero.def.experiencePerHP / 1000
	local expGainStr = string.format("exp %.1fk", expGainAvg)
	
	local quickNum = ""
	quickNum = string.format("#%d/%d  ", self.monsterIndex, listMonsterSize)
	
	self:set_text("str_infoplus", string.format("%sÆ: %s%s%s", quickNum, gemDrop, giveGold, expGainStr))
	
	self:StartAnimation("switchmonster_in")
end

function PreBattleMenu:OnAnimSwitchmonster_in()
	local def = self.monsterHero.def
	if def and def.voice then
		SoundSystem.Play(string.format("pq2audio/vox/%s_agro", def.voice))
	end
	
	self:EnableInput()
end


function PreBattleMenu:DetermineMonsterIndex()
	assert(self.monsterList)
	for i=1, #self.monsterList do
		if self.monsterList[i] == self.monsterHero.def.name then
			self.monsterIndex = i
			return i
		end
	end
	assert(string.format("No monster matched %s", tostring(self.monsterHero.def.name)))
	return
end

return ExportSingleInstance("PreBattleMenu")
