use_safeglobals()

local GOLD_COST		= 75
local MAX_ENTRIES	= 5

local MODE_MINUS	= 1
local MODE_PLUS		= 2
local MODE_APPLY	= 3

class "RespecMenu" (Menu);

function RespecMenu:__init()
	super()
	
	self:Initialize("Assets\\Screens\\RespecMenu.xml")
end

function RespecMenu:SetupAndOpen(hero)
	assert(hero)
	
	self.mode = MODE_MINUS
	
	self.hero = hero
	self.class = self.hero:GetAttribute("class")
	self.gold = self.hero:GetAttribute("gold")
	self.cost = 0
	
	self.minusStat = nil
	self.plusStat = nil
	
	self.currentStats = {}
	self.levelupStats = {}
	for i, stat in ipairs(STATS_ORDER) do
		self.currentStats[stat] = self.hero:GetStat(stat)
		self.levelupStats[stat] = self.hero:GetAttribute(stat)
	end
	
	self.preCurrentStats = table.clone(self.currentStats)
	self.preLevelupStats = table.clone(self.levelupStats)
	
	--[[ Useful for debugging
	LOG("~ 1: currentStats")
	table.log(self.currentStats)
	LOG("~ 2: levelupStats")
	table.log(self.levelupStats)
	--]]
	
	self:Open()
	self:UpdateDisplay()
	self:UpdateCostText()
end

function RespecMenu:UpdateDisplay()
	local strHint = ""
	if self.class == "Assassin" then
		strHint = "Hint: 6 36 36 21 6 \n"
	elseif self.class == "WarMage" then
		strHint = "Hint: 6 21 36 21 21 \n"
	elseif self.class == "Barbarian" then
		strHint = "Hint: 28 36 21 10 10 \n"
	elseif self.class == "Inquisitor" then
		strHint = "Hint: 3 36 28 10 3 \n"
	end
	if self.mode == MODE_MINUS then
		self:set_text_raw("str_msg", substitute(translate_text("[REMOVE_POINTS]"), strHint))
		self:set_text("str_gp_a", "[SELECT]")
		self:set_text("str_gp_b", "[CANCEL]")
		
		self:set_text("butt_a", "[SELECT]")
		self:set_text("butt_b", "[CANCEL]")
		
		self:hide_widget("str_gp_x")
		self:hide_widget("icon_gp_x")
		
		self:hide_widget("butt_x")
		
	elseif self.mode == MODE_PLUS then
		self:set_text_raw("str_msg", substitute(translate_text("[ADD_POINTS]"), strHint))
		self:set_text("str_gp_a", "[SELECT]")
		self:set_text("str_gp_b", "[BACK]")
		
		self:set_text("butt_a", "[SELECT]")
		self:set_text("butt_b", "[BACK]")
		
		self:hide_widget("str_gp_x")
		self:hide_widget("icon_gp_x")
		
		self:hide_widget("butt_x")
		
	elseif self.mode == MODE_APPLY then
		self:set_text("str_msg", "[TRANSFER_POINTS]")
		self:set_text("str_gp_a", "[DONE]")
		self:set_text("str_gp_b", "[BACK]")
		
		self:set_text("butt_a", "[DONE]")
		self:set_text("butt_b", "[BACK]")
		
		if IsGamepadActive() then
			self:activate_widget("str_gp_x")
			self:activate_widget("icon_gp_x")
			self:set_text("str_gp_x", "[TRANSFER]")
		else
			self:activate_widget("butt_x")
			self:set_text("butt_x", "[TRANSFER]")
		end
	end
	
	self:UpdateStatText()
end

function RespecMenu:UpdateStatText()
	for i, stat in ipairs(STATS_ORDER) do
		if self.minusStat and self.minusStat == stat then
			self:set_font(string.format("str_choice%d",  i), "font_uitext_red")
		elseif self.plusStat and self.plusStat == stat then
			self:set_font(string.format("str_choice%d",  i), "font_uitext_green")
		elseif (self.mode == MODE_MINUS or self.mode == MODE_APPLY) and not self:IsStatAboveMinimum(stat) then
			self:set_font(string.format("str_choice%d",  i), "font_uitext_gray")
		else
			self:set_font(string.format("str_choice%d",  i), "font_uitext")
		end
		
		self:set_text_raw(string.format("str_choice%d", i), string.format("%d %s", self.currentStats[stat], translate_text(string.format("[%s]", string.upper(STATS_ORDER[i])))))
	end
end

function RespecMenu:UpdateCostText()
	local cost_multiplier = 0
	for i, stat in ipairs(STATS_ORDER) do
		cost_multiplier = cost_multiplier + math.abs(self.levelupStats[stat] - self.preLevelupStats[stat])
	end
	
	self.cost = cost_multiplier * GOLD_COST
	
	if self.gold < self.cost then
		self:set_font("str_cost_value", "font_uitext_red")
	else
		self:set_font("str_cost_value", "font_uitext")
	end
	
	self:set_text_raw("str_cost_value", tostring(self.cost))
end

function RespecMenu:SelectEntry(index)
	index = index or 1
	
	SoundSystem.Play("pq2audio/ui/itemhover")
	self:set_image(string.format("icon_choice%d_bg", index), "img_LargeButton_on")
	self.selectedItem = index
end

function RespecMenu:OnOpen()
	if _G.Joystick then
		_G.Joystick.InitJoystick()
	end
	
	SoundSystem.Play("pq2audio/ui/menu_open")
	
	if IsGamepadActive() then
		self:SelectEntry()
	else
		self.selectedItem = nil
	end
	
	self.showInfo = false
	
	--SHOW TUTORIAL ONCE
	if not self.hero:GetRebuildTutorialSeen() then
		open_conversation_menu(
			"Assets/Conversations/REBUILDING_TUTORIAL.xml",
			self.hero:GetName(), 
			self.hero:GetConversationPortrait(), 
			self.hero:GetGenderForConversation(), 
			"","","", 
			function ()-- conversation finished, start game
				self.hero:SetRebuildTutorialSeen()
			end, 
			x or GetScreenWidth()*0.5,
			y or GetScreenHeight()*0.5
		)
	end
	
	return Menu.OnOpen(self)
end

function RespecMenu:OnRefresh()
	if IsGamepadActive() then
		self:activate_widget("str_gp_a")
		self:activate_widget("str_gp_b")
		self:activate_widget("str_gp_y")
		self:activate_widget("icon_gp_a")
		self:activate_widget("icon_gp_b")
		self:activate_widget("icon_gp_y")
		
		self:hide_widget("butt_a")
		self:hide_widget("butt_b")
		self:hide_widget("butt_x")
	else
		self:hide_widget("str_gp_a")
		self:hide_widget("str_gp_b")
		self:hide_widget("str_gp_x")
		self:hide_widget("str_gp_y")
		self:hide_widget("icon_gp_a")
		self:hide_widget("icon_gp_b")
		self:hide_widget("icon_gp_x")
		self:hide_widget("icon_gp_y")
		
		self:activate_widget("butt_a")
		self:activate_widget("butt_b")
		self:activate_widget("butt_x")
	end
	
	for i=1,MAX_ENTRIES do
		self:activate_widget(string.format("str_choice%d",  i))
		self:activate_widget(string.format("pad_choice%d", i))
		self:activate_widget(string.format("icon_choice%d_bg", i))
		self:set_image(string.format("icon_choice%d_bg", i), "img_LargeButton_409x64")
		self:activate_widget(string.format("icon_choice%d", i))
		self:set_image(string.format("icon_choice%d", i), string.format("img_levelup_%s", STATS_ORDER[i]))
	end
	
	self:set_text_raw("str_gold_value", tostring(self.gold))
	
	self:UpdateDisplay()
	self:UpdateCostText()
	
	if self.selectedItem then
		self:set_image(string.format("icon_choice%d_bg", self.selectedItem), "img_LargeButton_on")
	end
end

function RespecMenu:OnClose()
	if self.completedCallback then
		self.completedCallback()
		self.completedCallback = nil
	end
	
	return Menu.OnClose(self)
end

function RespecMenu:SetCompletedCallback(cb)
	self.completedCallback = cb
end

function RespecMenu:OnMouseLeftButton(pad_id, x_value, y_value)
	if self.mode ~= MODE_APPLY then
		for i=1,MAX_ENTRIES do
			if pad_id == self:get_widget_id(string.format("pad_choice%d", i)) then
				if self.selectedItem then
					self:set_image(string.format("icon_choice%d_bg", self.selectedItem), "img_LargeButton_409x64")
				end
				
				self:SelectEntry(i)
				
				return Menu.MESSAGE_HANDLED
			end
		end
	end
	
	return Menu.MESSAGE_HANDLED
end

function RespecMenu:OnMouseEnter(pad_id, x_value, y_value)
	x_value = self:get_widget_abs_x("pad_choice1")
	y_value = self:get_widget_abs_y("pad_choice1")
	
	for i=1,MAX_ENTRIES do
		if pad_id == self:get_widget_id(string.format("pad_choice%d", i)) then
			if self.selectedItem ~= i then
				SoundSystem.Play("pq2audio/ui/itemhover")
				self:set_image(string.format("icon_choice%d_bg", i), "img_LargeButton_409x64_hi")
			end
			
			self:ShowTooltip(STATS_ORDER[i], i, x_value, y_value)
			
			return Menu.MESSAGE_HANDLED
		end
	end
	
	return Menu.MESSAGE_HANDLED
end

function RespecMenu:OnMouseLeave(pad_id, x_value, y_value)
	for i=1,MAX_ENTRIES do
		if pad_id == self:get_widget_id(string.format("pad_choice%d", i)) then
			if self.selectedItem ~= i then
				self:set_image(string.format("icon_choice%d_bg", i), "img_LargeButton_409x64")
			end
			
			close_custompopup_menu(i)
			
			return Menu.MESSAGE_HANDLED
		end
	end
	
	return Menu.MESSAGE_HANDLED
end

function RespecMenu:ApplyRebuild()
	assert(self.plusStat)
	assert(self.minusStat)
	
	--[[ Useful for debugging
	LOG("~ A: currentStats")
	table.log(self.currentStats)
	LOG("~ B: levelupStats")
	table.log(self.levelupStats)
	--
	LOG("~ C: preCurrentStats")
	table.log(self.preCurrentStats)
	LOG("~ D: preLevelupStats")
	table.log(self.preLevelupStats)
	--]]
	
	self.hero:SetAttribute("gold", self.gold - self.cost)
	
	for i, stat in ipairs(STATS_ORDER) do
		self.hero:SetAttribute(stat, self.levelupStats[stat])
	end
	
	self.hero:CalculateStats()
	
	Achievements.AwardAchievement(1, "RESPEC")
end

function RespecMenu:OnGamepadJoystick(user_id, joystick_id, x_value, y_value)
	if _G.Joystick then
		_G.Joystick.OnGamepadJoystick(user_id, joystick_id, x_value, y_value)
	end
	return Menu.MESSAGE_HANDLED
end

function RespecMenu:OnGamepadDPad(user_id, pad_id, x_value, y_value)
	if x_value == 0 and y_value == 0 then
		return Menu.MESSAGE_HANDLED
	end
	
	local nextSelectedItem
	
	if self.selectedItem then
		if y_value > 0 and self.selectedItem > 1 then
			nextSelectedItem = self.selectedItem - 1
		elseif y_value < 0 and self.selectedItem < MAX_ENTRIES then
			nextSelectedItem = self.selectedItem + 1
		else
			return Menu.MESSAGE_HANDLED
		end
	else
		nextSelectedItem = 1
	end

	-- remove previous highlight
	if self.selectedItem then
		self:set_image(string.format("icon_choice%d_bg", self.selectedItem), "img_LargeButton_409x64")
	end
	
	-- remove previous tooltip
	if self.showInfo then
		if self.selectedItem then
			close_custompopup_menu(self.selectedItem)
		end
	end
	
	self:SelectEntry(nextSelectedItem)
	
	-- show new tooltip
	if self.showInfo then
		self:ShowTooltip(STATS_ORDER[nextSelectedItem], nextSelectedItem, x_value, y_value)
	end
	
	return Menu.MESSAGE_HANDLED
end

function RespecMenu:IsStatAboveMinimum(stat)
	assert(stat)
	
	return self.levelupStats[stat] > 0
end

function RespecMenu:IsGoldAboveCost()
	return self.cost <= self.gold
end

function RespecMenu:CheckStatsChanged()
	return self.cost > 0
end

function RespecMenu:FixSelection()
	if not IsGamepadActive() then
		if self.selectedItem then
			self:set_image(string.format("icon_choice%d_bg", self.selectedItem), "img_LargeButton_409x64")
			self.selectedItem = nil
		end
	end
end

function RespecMenu:MoveFoward()
	if self.mode == MODE_MINUS then
		if self.selectedItem then
			local stat = STATS_ORDER[self.selectedItem]
			if self:IsStatAboveMinimum(stat) then
				SoundSystem.Play("pq2audio/ui/menu_select")
				self.mode = MODE_PLUS
				self.minusStat = stat
				self:UpdateDisplay()
				self:FixSelection()
			else
				SoundSystem.Play("pq2audio/ui/illegal")
			end
		end
	elseif self.mode == MODE_PLUS then
		if self.selectedItem then
			local stat = STATS_ORDER[self.selectedItem]
			if stat ~= self.minusStat then
				SoundSystem.Play("pq2audio/ui/menu_select")
				self.mode = MODE_APPLY
				self.plusStat = stat
				self:UpdateDisplay()
				self:FixSelection()
			else
				SoundSystem.Play("pq2audio/ui/illegal")
			end
		end
	elseif self.mode == MODE_APPLY then
		local function ApplyAndClose()
			SoundSystem.Play("pq2audio/ui/gold")
			SoundSystem.Play("pq2audio/ui/award")
			self.mode = MODE_MINUS
			self:ApplyRebuild()
			-- save hero and close menu
			SCREENS.SaveMenu:Display(function () self:Close() end)
		end
		
		if self:IsGoldAboveCost() then
			if self:CheckStatsChanged() then
				ApplyAndClose()
			else
				open_yesno_menu("[LEAVE_MINIGAME]", "[REBUILD_LEAVE_CONFIRMATION]", function (option) 
					if not option then
						ApplyAndClose()
					end
				end, "[YES]", "[NO]")
			end
		else
			SoundSystem.Play("pq2audio/ui/illegal")
		end
	end
end

function RespecMenu:MoveBack()
	if self.mode == MODE_APPLY then
		SoundSystem.Play("pq2audio/ui/menu_deselect")
		self.mode = MODE_PLUS
		self.plusStat = nil
		self:UpdateDisplay()
	elseif self.mode == MODE_PLUS then
		SoundSystem.Play("pq2audio/ui/menu_deselect")
		self.mode = MODE_MINUS
		self.minusStat = nil
		self:UpdateDisplay()
		self:FixSelection()
	elseif self.mode == MODE_MINUS then
		SoundSystem.Play("pq2audio/ui/menu_deselect")
		self:Close()
	end
end

function RespecMenu:Transfer()
	if self.mode == MODE_APPLY then
		if self:IsStatAboveMinimum(self.minusStat) then
			close_custompopup_menu()
			SoundSystem.Play("pq2audio/ui/invswap")
			self.levelupStats[self.minusStat] = self.levelupStats[self.minusStat] - 1
			self.currentStats[self.minusStat] = self.currentStats[self.minusStat] - 1
			self.levelupStats[self.plusStat] = self.levelupStats[self.plusStat] + 1
			self.currentStats[self.plusStat] = self.currentStats[self.plusStat] + 1
			
			self:UpdateStatText()
			self:UpdateCostText()
			
			if self.showInfo and self.selectedItem then
				local widget_name = string.format("pad_choice%d", self.selectedItem)
				self:ShowTooltip(STATS_ORDER[self.selectedItem], self.selectedItem, self:get_widget_w(widget_name)/2, self:get_widget_h(widget_name)/2)
			end
		else
			SoundSystem.Play("pq2audio/ui/illegal")
		end
	end
end

function RespecMenu:OnButton(button_id, x_value, y_value)
	if button_id == self:get_widget_id("butt_a") then
		self:MoveFoward()
	elseif button_id == self:get_widget_id("butt_b") then
		self:MoveBack()
	elseif button_id == self:get_widget_id("butt_x") then
		self:Transfer()
	end
	return Menu.MESSAGE_HANDLED
end

function RespecMenu:OnKey(key)
	if key == Keys.SK_ESCAPE then
		self:MoveBack()
	end
	return Menu.MESSAGE_HANDLED
end

function RespecMenu:OnGamepadButton(user_id, button_id, up)
	if up == 0 then
		return Menu.MESSAGE_HANDLED
	end
	
	if button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_OKAY) then
		self:MoveFoward()
		
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CANCEL) then
		self:MoveBack()
		
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_LEFT) then
		self:Transfer()
		
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_TOP) then
		self.showInfo = not self.showInfo
		
		if self.showInfo then
			self:set_text("str_gp_y", "[MORE_INFO_HIDE]")
		else
			self:set_text("str_gp_y", "[MORE_INFO_SHOW]")
		end
		
		if self.selectedItem then
			if self.showInfo then
				local widget_name = string.format("pad_choice%d", self.selectedItem)
				self:ShowTooltip(STATS_ORDER[self.selectedItem], self.selectedItem, self:get_widget_w(widget_name)/2, self:get_widget_h(widget_name)/2)
			else
				close_custompopup_menu(self.selectedItem)
			end
		end
	end
	
	return Menu.MESSAGE_HANDLED
end

function RespecMenu:OnTimer(time)
	if _G.Joystick then
		_G.Joystick.OnTimer(self, time)
	end
	return Menu.MESSAGE_HANDLED
end

function RespecMenu:ShowTooltip(stat, index, x, y)
	assert(stat)
	assert(index)
	assert(x)
	assert(y)
	
	local showSpecialStatEffect = false
	if stat == "intelligence" then
		showSpecialStatEffect = true
	end
	
	local fmt, data = self:GetAttributeToolTipFormatAndData(stat, showSpecialStatEffect)
	
	if fmt and data then
		open_custompopup_menu(fmt, data, x, y, 4, 256, "RespecMenu", index)
	end
end

function RespecMenu:GetAttributeToolTipFormatAndData(attr, addSpecialEffect)
	local fmt = ""
	local data = ""
	
	local WIDTH = 416
	
	local HEADING_Y = 3
	local HEADING_HEIGHT = 40
	
	local SUBTITLE_Y = HEADING_Y + HEADING_HEIGHT
	local SUBTITLE_HEIGHT = 30
	
	local SPECIALEFFECT_Y = SUBTITLE_Y + SUBTITLE_HEIGHT
	local SPECIALEFFECT_HEIGHT = 32
	
	local VALUEEFFECT_Y = SUBTITLE_Y + SUBTITLE_HEIGHT
	local VALUEEFFECT_HEIGHT = 32
	
	if addSpecialEffect then
		VALUEEFFECT_Y = SPECIALEFFECT_Y + SPECIALEFFECT_HEIGHT
	end
	
	local RANKEFFECT_Y = VALUEEFFECT_Y + VALUEEFFECT_HEIGHT
	local RANKEFFECT_HEIGHT = 32
	
	local STARTMANA_Y = RANKEFFECT_Y + RANKEFFECT_HEIGHT
	local STARTMANA_HEIGHT = 32
	
	local EARNEDMANA_Y = STARTMANA_Y + STARTMANA_HEIGHT
	local EARNEDMANA_HEIGHT = 32
	
	local MAXMANA_Y = EARNEDMANA_Y + EARNEDMANA_HEIGHT
	local MAXMANA_HEIGHT = 32	
	
	local PAGE_OFFSET = MAXMANA_Y + MAXMANA_HEIGHT - HEADING_HEIGHT
	
	local EDGE_OFFSET = 3
	local BULLET_OFFSET = 10
	
	local currentScore = self.currentStats[attr]
	local currentRank = self.hero:GetStatPointsRanking(currentScore).rank
	local prevScore = currentScore - 1
	local prevRank = self.hero:GetStatPointsRanking(prevScore).rank
	local nextScore = currentScore + 1
	local nextRank = self.hero:GetStatPointsRanking(nextScore).rank
	
	local str_bullet = "img_levelup_" .. attr
	local upr_attr = string.upper(attr)
	local str_name   = translate_text(string.format("[%s]", upr_attr))
	
	-- Minigame bonus turns
	local str_special_effect
	if addSpecialEffect then
		str_special_effect   = translate_text(string.format("[%s_SPECIALEFFECT]", upr_attr))
	end
	
	local str_value_effect   = translate_text(string.format("[%s_VALUEFFECT]", upr_attr))
	local str_rank_effect    = translate_text(string.format("[%s_RANKEFFECT]", upr_attr))
	local str_mana1_effect   = translate_text(string.format("[%s_STARTMANAEFFECT]", upr_attr))
	local str_mana2_effect   = translate_text(string.format("[%s_EARNEDMANAEFFECT]", upr_attr))
	local str_mana3_effect   = translate_text(string.format("[%s_MAXMANAEFFECT]", upr_attr))
	
	local val01 = prevScore
	local val02 = prevRank
	local val03 = math.floor(prevRank/2)
	local val04 = math.floor(prevRank)
	local val05 = math.floor(prevRank*2)
	local val11 = currentScore
	local val12 = currentRank
	local val13 = math.floor(currentRank/2)
	local val14 = math.floor(currentRank)
	local val15 = math.floor(currentRank*2)
	local val21 = nextScore
	local val22 = nextRank
	local val23 = math.floor(nextRank/2)
	local val24 = math.floor(nextRank)
	local val25 = math.floor(nextRank*2)
	
	-- Title is the attribute name
	fmt = fmt .. "//SHADING/img_TextBackground_Red"
	fmt, data = TooltipHelpers.AddTitle(fmt, data, HEADING_Y+EDGE_OFFSET, "font_uitext", str_name)
	fmt = fmt .. "//SHADING/off"
	
	
	-- Show the previous levels amounts
	fmt = fmt .. "//SHADING/img_TextBackground_Purple"
	fmt, data = TooltipHelpers.AddTitle(fmt, data, SUBTITLE_Y, "font_uitext", substitute(translate_text("[LEVELUP_PREVIOUS]"), tostring(prevScore)))	-- raw
	fmt = fmt .. "//SHADING/off"
	
	if addSpecialEffect then
		-- spell penetration
		fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, SPECIALEFFECT_Y+4, 24, 24, str_bullet)
		fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, SPECIALEFFECT_Y, WIDTH-32, SPECIALEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_special_effect, val01))
	end
	
	-- N% chance for weapon critical
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, VALUEEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, VALUEEFFECT_Y, WIDTH-32, VALUEEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_value_effect, val01))
	
	-- +N Damage from Skull Gems
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, RANKEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, RANKEFFECT_Y, WIDTH-32, RANKEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_rank_effect, val02))
	
	-- +N Start Red mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, STARTMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, STARTMANA_Y, 238, STARTMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana1_effect, val03))
	
	-- +N Earned Red Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, EARNEDMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, EARNEDMANA_Y, WIDTH-32, EARNEDMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana2_effect, val04))
	
	-- +N Max Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, MAXMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, MAXMANA_Y, WIDTH-32, MAXMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana3_effect, val05))
	
	
	-- Show the current levels amounts
	fmt = fmt .. "//SHADING/img_TextBackground_Purple"
	fmt, data = TooltipHelpers.AddTitle(fmt, data, PAGE_OFFSET+SUBTITLE_Y, "font_uitext", substitute(translate_text("[LEVELUP_CURRENT]"), tostring(currentScore)))
	fmt = fmt .. "//SHADING/off"
	
	if addSpecialEffect then
		-- spell penetration
		fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+SPECIALEFFECT_Y+4, 24, 24, str_bullet)
		fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+SPECIALEFFECT_Y, WIDTH-32, SPECIALEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_special_effect, val11))
	end
	
	-- N% chance for weapon critical
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+VALUEEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+VALUEEFFECT_Y, WIDTH-32, VALUEEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_value_effect, val11))
	
	-- +N Damage from Skull Gems
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+RANKEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+RANKEFFECT_Y, WIDTH-32, RANKEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_rank_effect, val12))
	
	-- +N Start Red mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+STARTMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+STARTMANA_Y, 238, STARTMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana1_effect, val13))
	
	-- +N Earned Red Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+EARNEDMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+EARNEDMANA_Y, WIDTH-32, EARNEDMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana2_effect, val14))
	
	-- +N Max Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+MAXMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+MAXMANA_Y, WIDTH-32, MAXMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana3_effect, val15))
	
	
	-- Show the NEXT levels amounts
	fmt = fmt .. "//SHADING/img_TextBackground_Purple"
	fmt, data = TooltipHelpers.AddTitle(fmt, data, PAGE_OFFSET+PAGE_OFFSET+SUBTITLE_Y, "font_uitext", substitute(translate_text("[LEVELUP_NEXT]"), tostring(nextScore)))
	fmt = fmt .. "//SHADING/off"
	
	if addSpecialEffect then
		-- spell penetration
		fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+PAGE_OFFSET+SPECIALEFFECT_Y+4, 24, 24, str_bullet)
		fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+PAGE_OFFSET+SPECIALEFFECT_Y, WIDTH-32, SPECIALEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_special_effect, val21))
	end
	
	-- N% chance for weapon critical
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+PAGE_OFFSET+VALUEEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+PAGE_OFFSET+VALUEEFFECT_Y, WIDTH-32, VALUEEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_value_effect, val21))
	
	-- +N Damage from Skull Gems
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+PAGE_OFFSET+RANKEFFECT_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+PAGE_OFFSET+RANKEFFECT_Y, WIDTH-32, RANKEFFECT_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_rank_effect, val22))
	
	-- +N Start mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+PAGE_OFFSET+STARTMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+PAGE_OFFSET+STARTMANA_Y, 238, STARTMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana1_effect, val23))
	
	-- +N Earned Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+PAGE_OFFSET+EARNEDMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+PAGE_OFFSET+EARNEDMANA_Y, WIDTH-32, EARNEDMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana2_effect, val24))
	
	-- +N Max Mana
	fmt, data = TooltipHelpers.AddIcon(fmt, data, BULLET_OFFSET+4, PAGE_OFFSET+PAGE_OFFSET+MAXMANA_Y+4, 24, 24, str_bullet)
	fmt, data = TooltipHelpers.AddTextBlock(fmt, data, BULLET_OFFSET+32, PAGE_OFFSET+PAGE_OFFSET+MAXMANA_Y, WIDTH-32, MAXMANA_HEIGHT, "font_tooltip", "left", "vcenter", substitute(str_mana3_effect, val25))
	
	
	return fmt .. "//", data .. "//"
end

return ExportSingleInstance("RespecMenu")
