use_safeglobals()

class "SpellbookMenu" (Menu);

local tab
local TAB_CLASS  = 1
local TAB_QUEST  = 2
local TAB_FRIEND = 3

local MAX_SPELLS = 5
local LIST_SIZE = 12
local spell_colors = { "red", "green", "blue", "yellow", "black" }
local Listbox = require("Utils/IconListboxHelpers")
local LuaGamepad = require("Controls/Gamepad")
local MenuHelpers = require("Utils/MenuHelpers")

local gp_mode = { index = "equipped_spell1", show_info = false }
local gp_navigation = { 
	equipped_spell1 = { widget="icon_equipped_spell1_border",                       down="equipped_spell2", right="known_spell1", enabled=true },
	equipped_spell2 = { widget="icon_equipped_spell2_border", up="equipped_spell1", down="equipped_spell3", right="known_spell1", enabled=true },
	equipped_spell3 = { widget="icon_equipped_spell3_border", up="equipped_spell2", down="equipped_spell4", right="known_spell1", enabled=true },
	equipped_spell4 = { widget="icon_equipped_spell4_border", up="equipped_spell3", down="equipped_spell5", right="known_spell1", enabled=true },
	equipped_spell5 = { widget="icon_equipped_spell5_border", up="equipped_spell4",                         right="known_spell1", enabled=true },
	
	known_spell1    = { widget="icon_known_spell1_border",                          down="known_spell2",    left="equipped_spell1", enabled=true },
	known_spell2    = { widget="icon_known_spell2_border",    up="known_spell1",    down="known_spell3",    left="equipped_spell1", enabled=true },
	known_spell3    = { widget="icon_known_spell3_border",    up="known_spell2",    down="known_spell4",    left="equipped_spell1", enabled=true },
	known_spell4    = { widget="icon_known_spell4_border",    up="known_spell3",    down="known_spell5",    left="equipped_spell1", enabled=true },
	known_spell5    = { widget="icon_known_spell5_border",    up="known_spell4",    down="known_spell6",    left="equipped_spell1", enabled=true },
	known_spell6    = { widget="icon_known_spell6_border",    up="known_spell5",    down="known_spell7",    left="equipped_spell1", enabled=true },
	known_spell7    = { widget="icon_known_spell7_border",    up="known_spell6",    down="known_spell8",    left="equipped_spell1", enabled=true },
	known_spell8    = { widget="icon_known_spell8_border",    up="known_spell7",    down="known_spell9",    left="equipped_spell1", enabled=true },
	known_spell9    = { widget="icon_known_spell9_border",    up="known_spell8",    down="known_spell10",    left="equipped_spell1", enabled=true },
	known_spell10   = { widget="icon_known_spell10_border",    up="known_spell9",    down="known_spell11",     left="equipped_spell1", enabled=true },
	known_spell11   = { widget="icon_known_spell11_border",    up="known_spell10",    down="known_spell12",     left="equipped_spell1", enabled=true },
	known_spell12   = { widget="icon_known_spell12_border",    up="known_spell11",     									 left="equipped_spell1", enabled=true },
}

function SpellbookMenu:__init()
	super()
	self:Initialize("Assets\\Screens\\SpellbookMenu.xml")
end


function SpellbookMenu:OnOpen()
	if _G.Joystick then
		_G.Joystick.InitJoystick()
	end
	
	tab = TAB_CLASS
	
	self:SetupSpellLists()
	self:SetupListbox()
	
	self:SetListActive("known",false)
	self:SetListActive("equipped",true)
	
	self:MoveGamepadFocus(gp_mode.index)
	
	--SHOW TUTORIAL ONCE
	local hero = HeroManager:GetActiveHero()
	if not hero:GetAttribute("spellbookTutorialShown") and not is_conversation_menu_open() then
		open_conversation_menu(
			"Assets/Conversations/SPELLBOOK_TUTORIAL.xml",
			hero:GetName(), 
			hero:GetConversationPortrait(), 
			hero:GetGenderForConversation(), 
			"","","", 
			function ()-- conversation finished, start game
				hero:SetAttribute("spellbookTutorialShown",true)
			end, 
			x or GetScreenWidth()*0.5,
			y or GetScreenHeight()*0.5
		)
	end
	
	return Menu.OnOpen(self)
end

function SpellbookMenu:OnRefresh()
	if IsGamepadActive() then
		self:hide_widget("grp_arrows")
		self:activate_widget("str_gp_a")
		self:activate_widget("str_gp_y")
		self:activate_widget("str_gp_b")
		self:activate_widget("str_gp_x")
		
		self:activate_widget("icon_gp_a")
		self:activate_widget("icon_gp_y")
		self:activate_widget("icon_gp_b")
		self:activate_widget("icon_gp_l")
		self:activate_widget("icon_gp_r")
		self:activate_widget("icon_gp_x")
		self:hide_widget("butt_next")
		self:hide_widget("butt_prev")
		self:hide_widget("butt_close")
		self:set_widget_position("str_gp_l", self:get_widget_x("icon_gp_l") + 50,  self:get_widget_y("str_gp_l"))
		self:set_widget_position("str_gp_r", self:get_widget_x("icon_gp_r") - 110, self:get_widget_y("str_gp_r"))
		self:MoveGamepadFocus("equipped_spell1", "open")
		
		self:set_text("str_gp_y", "[MORE_INFO_SHOW]")
	else
		self:hide_widget("grp_arrows")
		self:hide_widget("str_gp_a")
		self:hide_widget("str_gp_y")
		self:hide_widget("str_gp_b")
		self:hide_widget("str_gp_x")
		
		self:hide_widget("icon_gp_a")
		self:hide_widget("icon_gp_y")
		self:hide_widget("icon_gp_b")
		self:hide_widget("icon_gp_l")
		self:hide_widget("icon_gp_r")
		self:hide_widget("icon_gp_x")
		
		self:set_widget_position("str_gp_l", self:get_widget_x("butt_prev") + 84,  self:get_widget_y("str_gp_l"))
		self:set_widget_position("str_gp_r", self:get_widget_x("butt_next") - 101, self:get_widget_y("str_gp_r"))
		
		self:activate_widget("butt_next")
		self:activate_widget("butt_prev")
		self:activate_widget("butt_close")
	end
	
	self:UpdateEquippedSpells()
	
	if tab == TAB_FRIEND then
		self:UpdateAvailableSpellsFriends()
	elseif tab == TAB_QUEST then
		self:UpdateAvailableSpellsQuest()
	else
		self:UpdateAvailableSpellsClass()
	end
	
	for i=1, MAX_SPELLS do
		self:hide_widget(string.format("icon_equipped_spell%d_border",i))
	end
	
	for i=1, LIST_SIZE do
		self:hide_widget(string.format("icon_known_spell%d_border",i))	
	end
end

function SpellbookMenu:OnClose()
	close_custompopup_menu()
	gp_mode.show_info = false
	
	Listbox.ClearList("spells")
	
	if self.completedCallback then
		self.completedCallback()
		self.completedCallback = nil
	end
	
	self.hero = nil
	
	return Menu.OnClose(self)
end

function SpellbookMenu:SetCompletedCallback(cb)
	self.completedCallback = cb
end

function SpellbookMenu:SetHero(hero)
	self.hero = hero
	self.hero_data = require("Hero/" .. tostring(hero:GetAttribute("class")))
end

function SpellbookMenu:MoveGamepadFocus(new_location, dir)
	if not IsGamepadActive() then
		return
	end
	
	if dir and (not gp_navigation[new_location].enabled) then
		return
	end
	
	SoundSystem.Play("pq2audio/ui/menu_select")
	
	local widget = gp_navigation[new_location].widget

	self:hide_widget(string.format("icon_%s_border",gp_mode.index))
	self:activate_widget(string.format("icon_%s_border",new_location))
	--self:set_widget_position("grp_arrows", self:get_widget_x(widget), self:get_widget_y(widget))
	if gp_mode.show_info then
		if self:OnMouseEnter(self:get_widget_id(string.format("pad_%s", new_location)), 0, 0) == Menu.MESSAGE_NOT_HANDLED then
			close_custompopup_menu()
		end
	end
	
	gp_mode.index = new_location
end

function SpellbookMenu:OnGamepadDPad(user_id, dpad_id, x_value, y_value)
	if x_value == 0 and y_value == 0 then
		return Menu.MESSAGE_NOT_HANDLED
	end
	
	LuaGamepad.NavigateUI(self, x_value, y_value, gp_navigation, gp_mode.index)
	return Menu.MESSAGE_HANDLED
end

function SpellbookMenu:OnGamepadJoystick(user_id, joystick_id, x_value, y_value)
	if _G.Joystick then
		_G.Joystick.OnGamepadJoystick(user_id, joystick_id, x_value, y_value)
	end
	return Menu.MESSAGE_HANDLED
end

function SpellbookMenu:OnButton(button_id, x_value, y_value)
	if button_id == self:get_widget_id("butt_close") then
		self:Close()
	elseif type(self.selected_slot) ~= "string" then
		if button_id == self:get_widget_id("butt_next") then
			self:ResetList()
			Listbox.NavigateList("spells", LIST_SIZE)
		elseif button_id == self:get_widget_id("butt_prev") then
			self:ResetList()
			Listbox.NavigateList("spells", -LIST_SIZE)
		elseif button_id == self:get_widget_id("butt_class") then
			self:ResetList()
			self:UpdateAvailableSpellsClass()
		elseif button_id == self:get_widget_id("butt_additional") then
			self:ResetList()
			self:UpdateAvailableSpellsQuest()
		elseif button_id == self:get_widget_id("butt_companion") then
			self:ResetList()
			self:UpdateAvailableSpellsFriends()
		end
	end
	return Menu.MESSAGE_HANDLED
end

function SpellbookMenu:OnKey(key)
	if key == Keys.SK_ESCAPE then
		self:Close()
	else
		return Menu.OnKey(self, key)
	end
	
	return Menu.MESSAGE_HANDLED
end

function SpellbookMenu:OnGamepadButton(user_id, button_id, up)
	if up == 0 then
		return Menu.MESSAGE_HANDLED
	end
	
	if button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_OKAY) then
		self:ProcessSpellSelection()
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CANCEL) then
		if self.selected_slot then
			self:DeselectSlot()
		else
			self:Close()
		end
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_TOP) then
		gp_mode.show_info = not gp_mode.show_info
		if gp_mode.show_info then
			if self:OnMouseEnter(self:get_widget_id(string.format("pad_%s", gp_mode.index)), 0, 0) == Menu.MESSAGE_NOT_HANDLED then
				close_custompopup_menu()
			end
			self:set_text("str_gp_y", "[MORE_INFO_HIDE]")
		else
			close_custompopup_menu()
			self:set_text("str_gp_y", "[MORE_INFO_SHOW]")
		end
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_LEFT) then
		SoundSystem.Play("pq2audio/ui/menu_open")
		if tab == TAB_CLASS then
			self:OnButton(self:get_widget_id("butt_additional"))
		elseif tab == TAB_QUEST then
			--self:set_widget_position("icon_gp_x", self:get_widget_x("icon_class_spells") + 67, self:get_widget_y("icon_class_spells") + 20)		
			self:OnButton(self:get_widget_id("butt_companion"))
		elseif tab == TAB_FRIEND then
			--self:set_widget_position("icon_gp_x", self:get_widget_x("icon_other_spells") + 67, self:get_widget_y("icon_other_spells") + 20)
			self:OnButton(self:get_widget_id("butt_class"))
		end
		if gp_mode.index:sub(1, 5) == "known" and gp_mode.show_info then	-- only update the popup on right hand side
			close_custompopup_menu()
			gp_mode.show_info = false
			self:OnGamepadButton(user_id, Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_CROSS_TOP), up)
		end
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_SHOULDER_LEFT) then
		if type(self.selected_slot) ~= "string" then
			close_custompopup_menu()
			gp_mode.show_info = false
			self:set_text("str_gp_y", "[MORE_INFO_SHOW]")
			Listbox.ScrollList("spells", -LIST_SIZE)
		end
	elseif button_id == Gamepad.MapGenericInput(user_id, GamepadInput.GAMEPAD_BUTTON_SHOULDER_RIGHT) then
		if type(self.selected_slot) ~= "string" then
			close_custompopup_menu()
			gp_mode.show_info = false
			self:set_text("str_gp_y", "[MORE_INFO_SHOW]")
			Listbox.ScrollList("spells", LIST_SIZE)
		end
	end
	return Menu.MESSAGE_HANDLED
end

function SpellbookMenu:OnMouseLeftButton(pad_id, x_value, y_value, up)
	if up then
		return Menu.MESSAGE_HANDLED
	end
	
	if	true or gp_navigation["equipped_spell1"].enabled then
		for i=1, MAX_SPELLS do
			local function SelectSpell()
				gp_mode.index = string.format("equipped_spell%d", i)
				self:ProcessSpellSelection()
			end
			
			if pad_id == self:get_widget_id(string.format("pad_equipped_spell%d", i)) then
				if type(self.selected_slot) == "number" then
					if self.selected_slot == i then
						self:DeselectSlot()
					else
						self:DeselectSlot()
						SelectSpell()
					end
				else 
					SelectSpell()
				end
			end
		end
	end
	
	
	if	gp_navigation["known_spell1"].enabled then
		for i=1, LIST_SIZE do
			if (pad_id == self:get_widget_id(string.format("pad_known_spell%d", i))) and type(self.selected_slot) ~= "string" then
				gp_mode.index = string.format("known_spell%d", i)
				self:ProcessSpellSelection()
			end
		end
	end
	
	return Menu.MESSAGE_HANDLED
end

function SpellbookMenu:ShowSpellTooltip(spell, x, y, id)
	local format, data = _G.GetSpellToolTipFormatAndData(spell)
	open_custompopup_menu(format, data, x, y, 4, 256, "SpellbookMenu", id)
	return
end

function SpellbookMenu:OnMouseEnter(pad_id, x_value, y_value)
	for i=1, MAX_SPELLS do
		local pad_name = string.format("pad_equipped_spell%d", i)
		if (pad_id == self:get_widget_id(pad_name)) and (i <= self.hero:NumAttributes("activeSpells")) then
			self:ShowSpellTooltip(self.hero:GetAttributeAt("activeSpells", i),
				self:get_widget_x(pad_name) + (self:get_widget_w(pad_name)/2),
				self:get_widget_y(pad_name) + self:get_widget_h(pad_name), i)
			return Menu.MESSAGE_HANDLED
		end
	end
	
	for i=1, LIST_SIZE do
		local pad_name = string.format("pad_known_spell%d", i)
		if (pad_id == self:get_widget_id(pad_name)) and Listbox.GetEntryFromField("spells", i) then
			self:ShowSpellTooltip(Listbox.GetEntryFromField("spells", i).spell,
				self:get_widget_x(pad_name) + (self:get_widget_w(pad_name)/2),
				self:get_widget_y(pad_name) + self:get_widget_h(pad_name), i)
			return Menu.MESSAGE_HANDLED
		end
	end
	
	return Menu.MESSAGE_NOT_HANDLED
end

function SpellbookMenu:OnMouseLeave(pad_id, x_value, y_value)
	for i=1, MAX_SPELLS do
		if (pad_id == self:get_widget_id(string.format("pad_equipped_spell%d", i))) then
			close_custompopup_menu(i)
		end
	end
	
	for i=1, LIST_SIZE do
		if (pad_id == self:get_widget_id(string.format("pad_known_spell%d", i))) then
			close_custompopup_menu(i)
		end
	end
	return Menu.MESSAGE_HANDLED
end


function SpellbookMenu:IndexToSlot(index)	
	--LOGF("SpellbookMenu:IndexToSlot(%s)",tostring(index))
	local slot = tostring(string.gsub(index,"(%D+)",""))
	--LOG(slot)
	return tonumber(slot)
end

function SpellbookMenu:DeselectSlot()
	LOGF("SpellbookMenu:DeselectSlot %s",tostring(self.selected_slot))
	self:SetListActive("known",false)
	self:SetListActive("equipped",true)
	
	scheduler:StopAllThreadsForObject(self)
	if self.thread_widget then
		self:set_alpha(self.thread_widget, 1.0)
		self:hide_widget(self.thread_widget)
	end
	self:MoveGamepadFocus(string.format("equipped_spell%d", self.selected_slot), nil)
	self.selected_slot = nil
	self.thread_object = nil
	self.thread_widget = nil
	
	self:set_text("str_gp_b", "[CLOSE]")
end

function SpellbookMenu:ResetJoystick()
	if _G.Joystick then
		_G.Joystick.InitJoystick()
	end
end

-- this function handles when the player selects either a spell or a slot
function SpellbookMenu:ProcessSpellSelection()
	SoundSystem.Play("pq2audio/ui/menu_open")
	
	local slot_number = self:IndexToSlot(gp_mode.index)
	if self.selected_slot then
		if type(self.selected_slot) == "number" then
			local selected_spell = Listbox.GetEntryFromField("spells", slot_number)
			if selected_spell then
				if selected_spell.equipped then
					self:ResetJoystick()
					open_message_menu("[EQUIP_FAIL_HEAD]", "[SPELL_FAIL_DESC_ALREADYEQUIPPED]", function() end)
				elseif (selected_spell.predicates) and (not CheckQuestPredicates(self.hero, selected_spell.predicates)) then
					self:ResetJoystick()
					open_message_menu("[EQUIP_FAIL_HEAD]", "[SPELL_FAIL_DESC_QUESTNYC]", function() end)
				elseif (selected_spell.level) and (selected_spell.level > self.hero:GetAttribute("level")) then
					self:ResetJoystick()
					open_message_menu("[EQUIP_FAIL_HEAD]", "[SPELL_FAIL_DESC_LOWLEVEL]", function() end)
				else
					if self.hero:NumAttributes("activeSpells") < self.selected_slot then
						self.hero:PushAttribute("activeSpells", Listbox.GetEntryFromField("spells", slot_number).spell)
					else
						self.hero:SetAttributeAt("activeSpells", self.selected_slot, selected_spell.spell)
					end
					self:UpdateEquippedSpells()
					self:MoveGamepadFocus(string.format("equipped_spell%d", self.selected_slot), nil)
					self:DeselectSlot()
				end
			end
		elseif type(self.selected_slot) == "string" then
			local equip_slot = slot_number
			if equip_slot > self.hero:NumAttributes("activeSpells") then
				self.hero:PushAttribute("activeSpells", self.selected_slot)
			else
				self.hero:SetAttributeAt("activeSpells", equip_slot, self.selected_slot)
			end
			self:UpdateEquippedSpells()
			self:DeselectSlot()
		end
	else
		if gp_mode.index:sub(1, 8) == "equipped" then			
			self:SetListActive("equipped",false)		
			self:SetListActive("known",true)
			self.selected_slot = slot_number
			self:MoveGamepadFocus("known_spell1", nil)
			
			self.thread_widget = string.format("icon_equipped_spell%d_border", self.selected_slot)
			scheduler:AddChildThread(self, function()
				while true do
					self:activate_widget(self.thread_widget)
					if self.selected_slot then
						MenuHelpers.FadeDownWidget(self, "SpellbookMenu", self.thread_widget, 1.0, 0.2, 0.6)
						scheduler:Sleep(100)
						MenuHelpers.FadeUpWidget(self, "SpellbookMenu", self.thread_widget, 0.2, 1.0, 0.6)
						scheduler:Sleep(100)
					else
						self:set_alpha(self.thread_widget, 1.0)
						self:hide_widget(self.thread_widget)
						coroutine.yield()
					end
				end
			end)
			
			self:set_text("str_gp_b", "[CANCEL]")
			
		elseif gp_mode.index:sub(1, 5) == "known" then	
			local spell_data = Listbox.GetEntryFromField("spells", slot_number)
			if not spell_data then
				return
			end
			
			if spell_data.equipped then
				self:ResetJoystick()
				open_message_menu("[EQUIP_FAIL_HEAD]", "[SPELL_FAIL_DESC_ALREADYEQUIPPED]", function() end)
			elseif (spell_data.predicates) and (not CheckQuestPredicates(self.hero, spell_data.predicates)) then
				self:ResetJoystick()
				open_message_menu("[EQUIP_FAIL_HEAD]", "[SPELL_FAIL_DESC_QUESTNYC]", function() end)
			elseif (spell_data.level) and (spell_data.level > self.hero:GetAttribute("level")) then
				self:ResetJoystick()
				open_message_menu("[EQUIP_FAIL_HEAD]", "[SPELL_FAIL_DESC_LOWLEVEL]", function() end)
			else
				self:SetListActive("known",false)
				self:SetListActive("equipped",true)
				
				self.thread_widget = string.format("icon_known_spell%d_border", slot_number)
				scheduler:AddChildThread(self, function()
					while true do
						self:activate_widget(self.thread_widget)
						if self.selected_slot then		
							MenuHelpers.FadeDownWidget(self, "SpellbookMenu", self.thread_widget, 1.0, 0.2, 0.6)
							scheduler:Sleep(100)
							MenuHelpers.FadeUpWidget(self, "SpellbookMenu", self.thread_widget, 0.2, 1.0, 0.6)
							scheduler:Sleep(100)
						else
							self:set_alpha(self.thread_widget, 1.0)
							self:hide_widget(self.thread_widget)
							coroutine.yield()
						end
					end
				end)
			
				self:set_text("str_gp_b", "[CANCEL]")
			
				self.selected_slot = spell_data.spell
				self:MoveGamepadFocus("equipped_spell1", nil)
			end
		end
	end
end

function SpellbookMenu:SetupSpellLists()
	self.class_spells = { }
	self.quest_spells = { }
	self.companions   = { }
	for _, spell in ipairs(self.hero_data.start_data.spells) do
		table.insert(self.class_spells, { spell = spell, level = 1 } )
	end
	for level, level_bonus in ipairs(self.hero_data.level_bonuses) do
		if level_bonus.spell then
			table.insert(self.class_spells, { spell = level_bonus.spell, level = level } )
		end
	end
	
	local trainable_data = require("Hero/TrainableSpells")
	for spell, predicates in pairs(trainable_data) do
		table.insert(self.quest_spells, { spell = spell, predicates = predicates } )
	end
	-- I have left the following in as a catch all 
	for i=1, self.hero:NumAttributes("knownSpells") do
		local curr_spell = self.hero:GetAttributeAt("knownSpells", i)
		local class_spell = false
		for _, spell_data in ipairs(self.class_spells) do
			if curr_spell == spell_data.spell then
				class_spell = true
			end
		end
		if not class_spell then
			table.insert(self.quest_spells, { spell = curr_spell })
		end
	end
	
	for i=1, self.hero:NumAttributes("friends") do
		local companion = self.hero:GetAttributeAt("friends", i)
		local friend_data = require("Friends/"..tostring(companion))
		table.insert(self.companions, { spell = friend_data.spell, portrait = friend_data.portrait } )		
	end
	
	self:UpdateEquippedStatus()
end

function SpellbookMenu:UpdateAvailableSpellsClass()
	tab = TAB_CLASS
	self:set_image("icon_class", "img_spellbook_class_hi")
	self:set_image("icon_additional", "img_spellbook_other")
	self:set_image("icon_companion", "img_spellbook_companion")
	--self:set_widget_position("icon_curr_tab", self:get_widget_x("icon_class_spells") + 67, self:get_widget_y("icon_class_spells") + 20)
	--self:set_widget_position("icon_gp_x", self:get_widget_x("icon_other_spells") + 67, self:get_widget_y("icon_other_spells") + 20)
	Listbox.ChangeList("spells", self.class_spells, 1)
end

function SpellbookMenu:UpdateAvailableSpellsQuest()
	tab = TAB_QUEST
	self:set_image("icon_class", "img_spellbook_class")
	self:set_image("icon_additional", "img_spellbook_other_hi")
	self:set_image("icon_companion", "img_spellbook_companion")
	--self:set_widget_position("icon_curr_tab", self:get_widget_x("icon_other_spells") + 67, self:get_widget_y("icon_other_spells") + 20)
	--self:set_widget_position("icon_gp_x", self:get_widget_x("icon_friend_spells") + 67, self:get_widget_y("icon_friend_spells") + 20)
	Listbox.ChangeList("spells", self.quest_spells, 1)
end

function SpellbookMenu:UpdateAvailableSpellsFriends()
	tab = TAB_FRIEND
	self:set_image("icon_class", "img_spellbook_class")
	self:set_image("icon_additional", "img_spellbook_other")
	self:set_image("icon_companion", "img_spellbook_companion_hi")
	--self:set_widget_position("icon_curr_tab", self:get_widget_x("icon_friend_spells") + 67, self:get_widget_y("icon_friend_spells") + 20)
	--self:set_widget_position("icon_gp_x", self:get_widget_x("icon_class_spells") + 67, self:get_widget_y("icon_class_spells") + 20)
	Listbox.ChangeList("spells", self.companions, 1)
end

function SpellbookMenu:SetListActive(list,active)
	
	local num = MAX_SPELLS
	if list == "known" then
		num = LIST_SIZE
	end
	for i=1, num do
		gp_navigation[string.format("%s_spell%d",list,i)].enabled = active
	end

end


function SpellbookMenu:ResetList()

	for i=1, LIST_SIZE do
		local icon_level_widget = string.format("icon_known_spell%d_level", i)
		self:hide_widget(icon_level_widget)
	end
			
end

function SpellbookMenu:UpdateEquippedSpells()
	local spell_colors = { "red", "green", "blue", "yellow", "black" }
	for i=1, MAX_SPELLS do
		if i <= self.hero:NumAttributes("activeSpells") then
			self:activate_widget(string.format("str_equipped_spell%d",  i))
			self:activate_widget(string.format("icon_equipped_spell%d", i))
			self:activate_widget(string.format("str_equipped_spell%d_level", i))
			
			local spell = self.hero:GetAttributeAt("activeSpells", i)
			local spell_data = assert(SPELLS[spell])
			self:set_text(string.format("str_equipped_spell%d",   i), string.format("[%s_NAME]", spell_data.id))
			self:set_image(string.format("icon_equipped_spell%d", i), spell_data.icon or "img_missing")
			
			self:set_text(string.format("str_equipped_spell%d_level", i), "")
			for _, class_spell in ipairs(self.class_spells) do
				if class_spell.spell == spell and class_spell.level then
					self:set_text_raw(string.format("str_equipped_spell%d_level", i), tostring(class_spell.level))
					break
				end
			end
			
			for _, color in ipairs(spell_colors) do
				if spell_data.cost[color] then
					self:activate_widget(string.format("icon_equipped_spell%d_%s", i, color))
					self:hide_widget(string.format("icon_equipped_spelloff%d_%s", i, color))
					self:activate_widget(string.format("str_equipped_spell%d_%s", i, color))
					self:set_text_raw(string.format("str_equipped_spell%d_%s", i, color), tostring(spell_data.cost[color]))
				else
					self:activate_widget(string.format("icon_equipped_spelloff%d_%s", i, color))
					self:hide_widget(string.format("icon_equipped_spell%d_%s", i, color))
					self:hide_widget(string.format("str_equipped_spell%d_%s", i, color))
				end
			end
		else
			self:hide_widget(string.format("str_equipped_spell%d", i))
			self:hide_widget(string.format("icon_equipped_spell%d", i))
			self:hide_widget(string.format("str_equipped_spell%d_level", i))
			
			self:hide_widget(string.format("str_equipped_spell%d_green", i))
			self:hide_widget(string.format("icon_equipped_spell%d_green", i))
			self:hide_widget(string.format("icon_equipped_spelloff%d_green", i))
			self:hide_widget(string.format("str_equipped_spell%d_red", i))
			self:hide_widget(string.format("icon_equipped_spell%d_red", i))
			self:hide_widget(string.format("icon_equipped_spelloff%d_red", i))
			self:hide_widget(string.format("str_equipped_spell%d_yellow", i))
			self:hide_widget(string.format("icon_equipped_spell%d_yellow", i))
			self:hide_widget(string.format("icon_equipped_spelloff%d_yellow", i))
			self:hide_widget(string.format("str_equipped_spell%d_blue", i))
			self:hide_widget(string.format("icon_equipped_spell%d_blue", i))
			self:hide_widget(string.format("icon_equipped_spelloff%d_blue", i))
			self:hide_widget(string.format("str_equipped_spell%d_black", i))
			self:hide_widget(string.format("icon_equipped_spell%d_black", i))
			self:hide_widget(string.format("icon_equipped_spelloff%d_black", i))
		end
	end
	
	self:UpdateEquippedStatus()
	
	Listbox.UpdateList("spells")
end

function SpellbookMenu:UpdateEquippedStatus()
	local activeSpells = CollectionToTable(self.hero, "activeSpells")
	for _, class_spell in ipairs(self.class_spells) do
		class_spell.equipped = false
		for _, spell in ipairs(activeSpells) do
			if class_spell.spell == spell then
				class_spell.equipped = true
			end
		end
	end
	for _, quest_spell in ipairs(self.quest_spells) do
		quest_spell.equipped = false
		for _, spell in ipairs(activeSpells) do
			if quest_spell.spell == spell then
				quest_spell.equipped = true
			end
		end
	end
	
	for _, companion in ipairs(self.companions) do
		companion.equipped = false
		for _, spell in ipairs(activeSpells) do
			if companion.spell == spell then
				companion.equipped = true
			end
		end
	end
end

function SpellbookMenu:SetupListbox()
	tab = TAB_CLASS
	self:set_color("icon_class_spells", 255, 255, 255)
	self:set_color("icon_other_spells", 100, 100, 100)
	self:set_color("icon_friend_spells", 100, 100, 100)
	
	Listbox.AddListbox("spells", self.class_spells, LIST_SIZE, 1)
	Listbox.SetCallback("spells", "SetData",
	function(entry, field)
		local spell_data = SPELLS[entry.spell]
		local str_widget = string.format("str_known_spell%d", field)
		local str_level_widget = string.format("str_known_spell%d_level", field)
		local icon_level_widget = string.format("icon_known_spell%d_level", field)
		local icon_widget = string.format("icon_known_spell%d", field)
		
		self:activate_widget(icon_widget)
		self:activate_widget(str_widget)
		self:activate_widget(str_level_widget)
		self:activate_widget(icon_level_widget)
		
		if entry.portrait then
			self:set_image(icon_widget, entry.portrait)
		else
			self:set_image(icon_widget, spell_data.icon)
		end
		self:set_text(str_widget, string.format("[%s_NAME]", spell_data.id))
		
		--LOGF("   SpellbookMenu:SetupListbox entry=%s field=%s entry.level=%s",tostring(entry),tostring(field),tostring(entry.level))
		if entry.level and entry.level ~= "" then
			self:set_text(str_level_widget, tostring(entry.level))
			--self:activate_widget(icon_level_widget)
			self:set_image(icon_level_widget, "img_spellbook_level")
		else
			self:set_text(str_level_widget, "")
			--self:hide_widget(icon_level_widget)
			self:set_image(icon_level_widget, "img_spellbook_blank")
		end
		
		--If Spell unavailable -gray text
		if tab == TAB_CLASS then
			if entry.level and entry.level > self.hero:GetAttribute("level") then
				self:set_font(str_widget, "font_script_lo")	
				self:set_font(str_level_widget, "font_script_gray")
			else
				self:set_font(str_widget, "font_script")
				self:set_font(str_level_widget, "font_script_white")
			end
		elseif tab == TAB_QUEST then
			if entry.predicates and not CheckQuestPredicates(self.hero, entry.predicates) then
				self:set_font(str_widget, "font_script_lo")	
			else
				self:set_font(str_widget, "font_script")
			end
		else--TAB_FRIENDS
				self:set_font(str_widget, "font_script")		
		end
		--if equipped - highlight
		if entry.equipped then
			self:set_font(str_widget, "font_script_hi")
			self:set_font(str_level_widget, "font_script_white")
		end
		
		for _, color in ipairs(spell_colors) do
			if spell_data.cost[color] then
				self:activate_widget(string.format("icon_known_spell%d_%s", field, color))
				self:activate_widget(string.format("str_known_spell%d_%s", field, color))
				self:set_text_raw(string.format("str_known_spell%d_%s", field, color), tostring(spell_data.cost[color]))
			else
				self:hide_widget(string.format("icon_known_spell%d_%s", field, color))
				self:hide_widget(string.format("str_known_spell%d_%s", field, color))
			end
		end
		
	end)
	Listbox.SetCallback("spells", "NoData",
	function(field)
		self:hide_widget(string.format("icon_known_spell%d", field))
		self:hide_widget(string.format("str_known_spell%d", field))
		self:hide_widget(string.format("str_known_spell%d_level", field))
		self:hide_widget(string.format("icon_known_spell%d_level", field))
		for _, color in ipairs(spell_colors) do
			self:hide_widget(string.format("icon_known_spell%d_%s", field, color))
			self:hide_widget(string.format("str_known_spell%d_%s", field, color))
		end
	end)
	Listbox.SetCallback("spells", "UpdateDisplay", 
	function()
		local gpActive = false
		if IsGamepadActive() then
			gpActive = true
		end
		local curr_page = math.max(1, math.ceil(Listbox.GetCurrentIndex("spells")/Listbox.GetVisibleLength("spells")))
		local max_pages = math.max(1, math.ceil(Listbox.GetListSize("spells")/Listbox.GetVisibleLength("spells")))
		if curr_page == 1 then
			self:hide_widget("str_gp_l")
			if gpActive then
				self:hide_widget("icon_gp_l")
			else
				self:hide_widget("butt_prev")
			end
		else
			self:activate_widget("str_gp_l")
			if gpActive then
				self:activate_widget("icon_gp_l")
			else
				self:activate_widget("butt_prev")
			end
		end
		if curr_page == max_pages then
			self:hide_widget("str_gp_r")
			if gpActive then
				self:hide_widget("icon_gp_r")
			else
				self:hide_widget("butt_next")
			end
		else
			self:activate_widget("str_gp_r")
			if gpActive then
				self:activate_widget("icon_gp_r")
			else
				self:activate_widget("butt_next")
			end
		end
	end)
	Listbox.NavigateTo("spells", 1)
	Listbox.UpdateList("spells")
end

function SpellbookMenu:OnTimer(time)
	if _G.Joystick then
		_G.Joystick.OnTimer(self, time)
	end
	return Menu.MESSAGE_HANDLED
end

return ExportSingleInstance("SpellbookMenu")