local Blackjack = {}
local Spell = require("Spells/Spell")

--[[

You Stun your opponent for their next 3 turns. Strike = Double Damage 
to cast. Does not end your turn.

]]

Blackjack.id			= "BLACKJACK"
Blackjack.icon			= "img_spell_blackjack"
Blackjack.short_name	= "BLKJCK"
Blackjack.sound	= "sp_blackjack"
Blackjack.sound_ds = "snd_sp_stun"
Blackjack.sound_bank_ds = "BANK_SP_STUN"
Blackjack.dietfxParticle	= "magicfireball"
Blackjack.dietfxTexture	= "fireball"
Blackjack.is_stat_logged = true
Blackjack.cost			= {
	red					= 10,
	green				= 8,
}
Blackjack.effects = {
	{"[BLACKJACK_EFFECT0_HEAD]", "[BLACKJACK_EFFECT0_BODY]"},
}
Blackjack.title = Spell:GetTitleTag(Blackjack)
Blackjack.subtitles = {
	Spell:GetEffectTag(Blackjack, 0),
}
Blackjack.cooldown = 4
Blackjack.cooldownForAI = 4

function Blackjack:Apply(params)
	return function ()
		Spell:Pause(Spell.SPELL_INTRO_PAUSE)
		Spell:ApplyStatusEffectTo(params.target, params, self, "Stun", 6)
		Spell:AwardBonusMove(params)
		Spell:Pause(Spell.SPELL_MIN_DURATION)
		Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	end
end


local EXPLOSION_DURATION = 1000
local VARIATION = 14
-- define update function
local function shrink(pdef_a, d_t)
	for i, def in ipairs(pdef_a) do
		local var = VARIATION - VARIATION* d_t/EXPLOSION_DURATION
		var = var > 0 and var or 0
		def:SetVelocityVariation(var)
	end
end

function Blackjack:Notify(grid,notify)
	local heading, text, startTime = Spell:ShowStandardSpellHeader(self.title)
	
	local duration = 1000
	local client = notify:GetAttribute("client")
	
	--CircleButton	
	local circle_particle = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("LightningPathYellow"))
	circle_particle:SetLifeCycle(duration/2)
	circle_particle:SetLifetime(duration)
	circle_particle:SetAlpha(0.0)
	circle_particle:SetTargetAlpha(0.4)
	circle_particle:SetAnimationStart(0)
	Spell:CircleWidget(client,Spell:GetSpellButtonWidgetName(notify),{circle_particle},duration,12)
	
	--Fire particles
	local particle = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("RuneSpellYellow"))
	particle:SetLifeCycle(100)
	particle:SetSize(0.4)
	particle:SetAnimateSize(false)
	
	local delay = 1000
	
	
	local widgetsList = {{1,Spell:GetSpellButtonWidgetName(notify),0,0};
		  {2,"icon_portrait",0,0};
		};
	local path = Spell:WidgetPath(nil,notify,widgetsList,0,0)
	path:SetAttribute("Duration",duration)
	Spell:AttachParticleMotionFragment(path, {particle}, nil,delay)
	
	
	
	
	--Explosion masked by portrait	
	local opClient = Spell:GetOpposingClient(client)
	local x,y = Spell:GetWidgetTargetPosition(opClient,"icon_portrait")
	local flip = x > GetScreenWidth()/2
	
	path = MovementManager:Construct("RoundedNonuniformSpline")
	path:SetAttribute("Duration",EXPLOSION_DURATION)
	for i=1, 30 do
		Spell:PushPosition(path,x+math.random(0,1),y+math.random(0,1))
		Spell:PushVelocity(path,0,0)
	end
	Spell:PushPosition(path,x,y)
	Spell:PushVelocity(path,0,0)
	
	particle = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("YellowExplosion"))
	particle:SetNumParticlesToRelease(80)
	particle:SetReleaseInterval(10)
	particle:SetLifeCycle(900)
	particle:SetMaxParticles(10000)
	particle:SetTotalParticles(0)
	particle:SetAnimationStart(0)
	particle:SetVelocityVariation(VARIATION)
	particle:SetLifetime(EXPLOSION_DURATION+duration+delay)
	
	local fragment = GameObjectManager:ConstructLocal("BGMF")
	local view = SCREENS.EffectsMenu:GetWorld()
	view:AddChild(fragment)
	local portrait = GameObjectViewManager:Construct("GraphicAsset", tostring(SCREENS[opClient].portrait))
	fragment:SetView(portrait)
	local pw = portrait:GetExtentsX()
	local ww = SCREENS[opClient]:get_widget_w("icon_portrait")
	portrait:SetScale(ww/pw)
	fragment:SetPos(x,y)
	fragment.particles = ParticleLibrary.CreateParticleSystem(particle)
	
	-- Duplicate frame to Mask shaky portrait
	fragment.frame = GameObjectManager:ConstructLocal("BGMF")
	fragment.frame:SetPos(Spell:GetWidgetTargetPosition(opClient,"icon_portrait_frame"))
	fragment:AddChild(fragment.frame)
	local frame = "img_battlegame_portrait_frame"
	if GetScreenOffset() > 0 then
		frame = "img_portraitframe_ws"
	end
	frame = GameObjectViewManager:Construct("GraphicAsset", frame)	
	frame:SetScale(ww/pw)
	frame:SetSortingValue(-4)
	fragment.frame:SetView(frame)
	
	
	local mask = GameObjectViewManager:Construct("GraphicAsset", tostring(SCREENS[opClient].portrait))
	if flip then
		portrait:SetFlip(true)
		mask:SetFlip(true)
	end
	fragment:AddOverlay("mask",mask,0,0)
	mask:SetScale(ww/pw)
	mask:SetStencilWriteOps(StencilOp.Replace, StencilOp.Replace, StencilOp.Replace, 1)
	mask:SetStencilRefValue(1)

	fragment.particles:SetStencilFunction(StencilFunc.NotEqual, 1)
	fragment.particles:SetStencilRefValue(1)
	
	
	scheduler:AddChildThread(fragment, function ()
		scheduler:Sleep(delay+duration)
		fragment:SetMovementController(path)
		fragment:AddParticlesPos(fragment.particles, 0,0, 0)
		local start_time = GetGameTime()
		while true do
			local d_t = GetGameTime() - start_time
			shrink({particle}, d_t)
			coroutine.yield()
		end
	
	end, "Particle Thread")
	
	
	
	
	
	
	Spell:Pause(Spell.SPELL_INTRO_PAUSE)
	Spell:ShowStandardEffectSubtitle(self.subtitles[1], Spell.SPELL_MIN_DURATION)
	Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	Spell:RemoveStandardSpellHeader(heading, text, startTime)
end

return Blackjack
