local Confuse = {}
local Spell = require("Spells/Spell")

--[[

Places the Confuse! status effect on your opponent, drastically reducing their 
effectiveness on the battlefield. On their next turn, the opponent gains 1 
Mana per Mana Gem match made, rather than the matched amount. This spell can 
be extended by 1 turn for each turn you match Yellow Gems on the board. Does not end your turn.

]]

Confuse.id	= "CONFUSE"
Confuse.icon	= "img_spell_confuse"
Confuse.short_name = "CONFUS"
Confuse.sound	= "sp_confuse"
Confuse.sound_ds = "snd_sp_cursedark"
Confuse.sound_bank_ds = "BANK_SP_CURSEDARK"
Confuse.dietfxParticle	= "magicexplosion"
Confuse.dietfxTexture	= "stone"
Confuse.is_stat_logged = true
Confuse.cost	= {
	green	= 6,
	blue	= 5,
}
Confuse.effects = {
	{"[CONFUSE_EFFECT0_HEAD]", "[CONFUSE_EFFECT0_BODY]"},
}
Confuse.title = Spell:GetTitleTag(Confuse)
Confuse.subtitles = {
	Spell:GetEffectTag(Confuse, 0),
}
Confuse.cooldownForAI = 5
Confuse.cooldown = 5

function Confuse:Apply(params)

	local duration = 4
	
	return function ()
		Spell:Pause(Spell.SPELL_INTRO_PAUSE)
		Spell:ApplyStatusEffectTo(params.target, params, self, "ManaMutate", duration*2, 0, 1, "AllAndPower")
		Spell:Pause(Spell.SPELL_MIN_DURATION)
		Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
		Spell:AwardBonusMove(params)
	end
end




function Confuse:Notify(grid,notify)
	local heading, text, startTime = Spell:ShowStandardSpellHeader(self.title)
	
	local duration = 2400
	local client = notify:GetAttribute("client")
	
	--CircleButton	
	local circle_particle = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("LightningPathPurple"))
	--circle_particle:SetReleaseInterval(1)
	circle_particle:SetLifeCycle(duration/2)
	circle_particle:SetColor(0.5,0.05,0.7,0.2)
	circle_particle:SetTargetColor(0.5,0.05,0.7,0.05)
	circle_particle:SetAnimationStart(duration/4)
	Spell:CircleWidget(client,Spell:GetSpellButtonWidgetName(notify),{circle_particle},duration,12)
	
	
	
	--Spinning Symbol
	local widgetsList = { {2,"icon_portrait",0,0};
			  {2,"icon_portrait",0,4};
			};
	local path = Spell:WidgetPath(nil,notify,widgetsList,0,0)
	path:SetAttribute("Duration",duration)
	
	local particle = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("SpinningSymbol"))
	particle:SetAnimationStart(700)
	particle:SetLifeCycle(duration)
	particle:SetSize(3)
	particle:SetTargetSize(5)
	particle:SetColor(0.5,0.05,0.8,1)
	particle:SetTargetColor(0.5,0.05,0.8,0)
	particle:SetAngularVelocity(-3)
	Spell:AttachParticleMotionFragment(path, {particle}, nil)
	
	Spell:Pause(Spell.SPELL_INTRO_PAUSE)
	Spell:ShowStandardEffectSubtitle(self.subtitles[1], Spell.SPELL_MIN_DURATION)
	Spell:RemoveStandardSpellHeader(heading, text, startTime)
end

return Confuse
