local MirrorShield = {}
local Spell = require("Spells/Spell")

MirrorShield.id   = "MIRRORSHIELD"
MirrorShield.cost = {
	green  = 8,
	blue   = 4,
}
MirrorShield.warmageSpellIndex = 15
MirrorShield.icon = "img_spell_mirror_shield"
MirrorShield.short_name	= "MIRROR"
MirrorShield.sound	= "sp_mirrorshield"
MirrorShield.sound_ds = "snd_sp_manablock"
MirrorShield.sound_bank_ds = "BANK_SP_MANABLOCK"
MirrorShield.dietfxParticle	= "magicwall"
MirrorShield.dietfxTexture	= "shield"
MirrorShield.is_stat_logged  = true
MirrorShield.effects = {
	{"[MIRRORSHIELD_EFFECT0_HEAD]", "[MIRRORSHIELD_EFFECT0_BODY]"},
}

MirrorShield.title = Spell:GetTitleTag(MirrorShield)
MirrorShield.subtitles = {
	Spell:GetEffectTag(MirrorShield, 0),
}
MirrorShield.cooldownForAI = 10
MirrorShield.cooldown = 5

--[[

Causes half of any damage done to you to be reflected at your attacker, with 
the shield staying up for every subsequent turn you can make at least one 
Blue Gem match (or until dispelled). Does not end your turn.

]]

function MirrorShield:Apply(params)

	local numTurns = 10 + math.floor(params.source.state.blue/8)
	local defBonus = params.target.state:GetDefence()

	return function ()
		Spell:Pause(Spell.SPELL_INTRO_PAUSE)
		--Spell:ApplyStatusEffectTo(params.source, params, self, "ReflectDamage", 2*numTurns+1, 0, 50)
		if defBonus > 0 then 
			Spell:ApplyStatusEffectTo(params.source, params, self, "ModifyDefense", numTurns*2+1, 0, defBonus)
			Spell:ApplyStatusEffectTo(params.target, params, self, "ModifyDefense", numTurns*2+1, 0, -defBonus)
		end
		Spell:AwardBonusMove(params)
		Spell:Pause(Spell.SPELL_MIN_DURATION)
		Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	end
end

function MirrorShield:Notify(grid,notify)
	local heading, text, startTime = Spell:ShowStandardSpellHeader(self.title)
	
	
	Spell:AddParticleTrail("Paths.Blue1", "LongPathBlue", 0, 0,1500)
	Spell:AddParticleTrail("Paths.Blue2", "LongPathBlue", 0, 0,1500)
	Spell:AddParticleTrail("Paths.Blue3", "LongPathBlue", 0, 0,1500)
	Spell:AddParticleTrail("Paths.Shield1", "LongPathBlue", 0, 0,2000)
	Spell:AddParticleTrail("Paths.Shield2", "LongPathBlue", 0, 0,2000)
	Spell:AddParticleTrail("Paths.Shield3", "LongPathBlue", 0, 0,2000)
	
	Spell:Pause(Spell.SPELL_INTRO_PAUSE)
	Spell:ShowStandardEffectSubtitle(self.subtitles[1], Spell.SPELL_MIN_DURATION)
	Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	Spell:RemoveStandardSpellHeader(heading, text, startTime)
end

return MirrorShield
