local Rush = {}
local Spell = require("Spells/Spell")

--[[

Destroys all skulls with no effect. Enemy is stunned for 1 turn + 1 turn per 5 skulls destroyed. Does not end your turn.

]]

Rush.id			= "RUSH"
Rush.cost		= {
	yellow			= 7,
	green			= 3,
}
Rush.icon		= "img_spell_rush"
Rush.short_name	= "RUSH"
Rush.sound	= "sp_rush"
Rush.sound_ds = "snd_sp_stun"
Rush.sound_bank_ds = "BANK_SP_STUN"
Rush.dietfxParticle	= "magicfireball"
Rush.dietfxTexture	= "storm"
Rush.is_stat_logged  = true
Rush.effects 	= {
	{"[RUSH_EFFECT0_HEAD]", "[RUSH_EFFECT0_BODY]"}
}
Rush.title 	= Spell:GetTitleTag(Rush)
Rush.subtitles = {
	Spell:GetEffectTag(Rush, 0),
}
Rush.cooldownForAI = 7
Rush.cooldown = 7

function Rush:Apply(params)
	
	local yellowMana = params.source.state.yellow
	local numTurns = 2 + math.floor(yellowMana/7)

	
	return function ()
		Spell:Pause(Spell.SPELL_INTRO_PAUSE*5)--Extra Delay for particle effect
		-- Stun Enemy
		Spell:ApplyStatusEffectTo(params.target, params, self, "Stun", 2*numTurns)
		Spell:AwardBonusMove(params)
		Spell:Pause(Spell.SPELL_MIN_DURATION)
		Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	end
end

function Rush:Notify(grid,notify)
	local heading, text, startTime = Spell:ShowStandardSpellHeader(self.title)
	
	local client = notify:GetAttribute("client")
	local opClient = Spell:GetOpposingClient(client)
	
	local widgetsList,path
	local particle
	
	local duration = 1500 
	local delay = 0
	
	--Circle PortraitFrame
	SCREENS[client]:StopTurnIndicator()
	local circle_particle = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("LightningPathYellow"))
	circle_particle:SetReleaseInterval(1)
	circle_particle:SetLifeCycle(duration/2)
	circle_particle:SetAlpha(0.4)
	circle_particle:SetTargetAlpha(0.05)
	circle_particle:SetSize(0.15)
	circle_particle:SetTargetSize(0.6)
	--circle_particle:SetAnimateSize(false)
	circle_particle:SetAnimationStart(duration/4)
	Spell:CircleWidget(client,"icon_portrait_frame",{circle_particle},duration,4)
	
	
	delay = duration+500
	duration = 1600
	
	--Arc rushes towards opponent
	local start_x,start_y = Spell:GetWidgetTargetPosition(client, "icon_portrait_frame")
	local dir = 1
	if start_x > _G.GetScreenWidth()/2 then
		dir=-1
	end
	local height = SCREENS[client]:get_widget_h("icon_portrait_frame")
	start_x = start_x + (SCREENS[client]:get_widget_w("icon_portrait_frame")/2 - 10)*dir
	start_y = start_y - height/2
	
	local offsets = {
							{0,0},
							{10,22},
							{18,44},
							{24,66},
							{29,88},
							{33,110},
							{36,132},
							{36,154},
							{35,176},
							{27,198},
							{22,218},
							{16,234},
						}
	
	
	local endVel = 2
	local end_x, end_y = Spell:GetWidgetTargetPosition(opClient, "icon_portrait_frame")
	local distance = end_x - start_x 
	
	particle = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("FireBallSmall"))	
	particle:SetOffsetVariation(0.1)
	particle:SetReleaseInterval(2)
	particle:SetAnimateSize(false)
	
	
	particle:SetSize(0.65)
	particle:SetLifeCycle(200)
	
	local num = #offsets
	
	for i,offset in ipairs(offsets) do
		offset[2] = (i/num) * height
		local p = ParticleLibrary.CloneDescription(particle)
		path = MovementManager:Construct("RoundedNonuniformSpline")
		path:SetAttribute("Duration",duration)
		Spell:PushPosition(path,start_x+offset[1]*dir, start_y+offset[2])
		Spell:PushVelocity(path,0,0)
		Spell:PushPosition(path,start_x+(offset[1])*dir+distance, start_y+offset[2])
		Spell:PushVelocity(path,endVel*dir,0)
		Spell:AttachParticleMotionFragment(path, {p}, nil,delay)
	end
	
	
	delay = duration+delay
	duration = 1000
	
	--explosions on portrait
	widgetsList = {
			  {2,"icon_portrait",-100,60};
			  {2,"icon_portrait",-100,-60};
			  {2,"icon_portrait",100,60};
			  {2,"icon_portrait",100,-60};
			  {2,"icon_portrait",-100,60};
			};
	path = Spell:WidgetPath(nil,notify,widgetsList,1,1.0)
	path:SetAttribute("Duration",duration)
	local particle1 = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("YellowExplosion"))
	local particle2 = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("GemYellowExp04"))
	local particles = {particle1,particle2}
	particle2:SetNumParticlesToRelease(10)
	for _,p in ipairs(particles) do
		p:SetLifetime(0)
		p:SetReleaseInterval(200)
		p:SetMaxParticles(2000)
		p:SetTotalParticles(0)
	end
	Spell:AttachParticleMotionFragment(path, particles, nil,delay)

	--Shake Portrait
	local x,y = Spell:GetWidgetTargetPosition(opClient,"icon_portrait")
	local flip = x > GetScreenWidth()/2
	
	path = MovementManager:Construct("RoundedNonuniformSpline")
	path:SetAttribute("Duration",duration)
	local jitter = {{0,1},{-1,0},{1,1},{0,0}}
	local j=1
	for i=1, 30 do
		local offset = jitter[j]
		j = j + 1
		if j > 4 then
			j=1
		end
		Spell:PushPosition(path,x+offset[1],y+offset[2])
		Spell:PushVelocity(path,0,0)
	end
	Spell:PushPosition(path,x,y)
	Spell:PushVelocity(path,0,0)
	
	local fragment = GameObjectManager:ConstructLocal("BGMF")
	local view = SCREENS.EffectsMenu:GetWorld()
	view:AddChild(fragment)
	fragment:SetPos(x,y)
	local portrait = GameObjectViewManager:Construct("GraphicAsset", tostring(SCREENS[opClient].portrait))
	fragment:SetView(portrait)
	if flip then
		portrait:SetFlip(true)
	end
	local pw = portrait:GetExtentsX()
	local ww = SCREENS[client]:get_widget_w("icon_portrait")
	portrait:SetScale(ww/pw)
	portrait:SetSortingValue(-3)
	
	-- Duplicate frame to Mask shaky portrait
	fragment.frame = GameObjectManager:ConstructLocal("BGMF")
	fragment.frame:SetPos(Spell:GetWidgetTargetPosition(opClient,"icon_portrait_frame"))
	fragment:AddChild(fragment.frame)
	local frame = "img_battlegame_portrait_frame"
	if GetScreenOffset() > 0 then
		frame = "img_portraitframe_ws"
	end
	frame = GameObjectViewManager:Construct("GraphicAsset", frame)	
	frame:SetScale(ww/pw)
	frame:SetSortingValue(-4)
	fragment.frame:SetView(frame)
	
	scheduler:AddChildThread(fragment, function ()
		scheduler:Sleep(delay)		
		fragment:SetMovementController(path)
		local player = opClient:GetAttribute("player_id")
		if player >= 1 and player <= 4 then
			AddRumble(PlayerToUser(player), 0.75, duration)
		end
	end, "Particle Thread")
	
	
	
	Spell:Pause(Spell.SPELL_INTRO_PAUSE)
	Spell:ShowStandardEffectSubtitle(self.subtitles[1], Spell.SPELL_MIN_DURATION)
	Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	Spell:RemoveStandardSpellHeader(heading, text, startTime)
end

return Rush
