local SkullCrusher = {}
local Spell = require("Spells/Spell")

--[[

Destroys all skulls with no effect. Enemy is stunned for 1 turn + 1 turn per 5 skulls destroyed. Does not end your turn.

]]

SkullCrusher.id			= "SKULLCRUSHER"
SkullCrusher.cost		= {
	red				= 12,
	green			= 6,
}
SkullCrusher.icon		= "img_spell_skull_crusher"
SkullCrusher.short_name	= "SCRUSH"
SkullCrusher.sound	= "sp_skullcrusher"
SkullCrusher.sound_ds = "snd_sp_destroy"
SkullCrusher.sound_bank_ds = "BANK_SP_DESTROY"
SkullCrusher.dietfxParticle	= "magicfireball"
SkullCrusher.dietfxTexture	= "Skull"
SkullCrusher.is_stat_logged  = true
SkullCrusher.effects 	= {
	{"[SKULLCRUSHER_EFFECT0_HEAD]", "[SKULLCRUSHER_EFFECT0_BODY]"}
}
SkullCrusher.title 	= Spell:GetTitleTag(SkullCrusher)
SkullCrusher.subtitles = {
	Spell:GetEffectTag(SkullCrusher, 0),
}
SkullCrusher.cooldownForAI = 3
SkullCrusher.cooldown = 3

function SkullCrusher:Apply(params)
	
	local numSkulls = params.grid:CountByName("Skull") + params.grid:CountByName("Skull5")
	local numTurns = 1 + math.floor(numSkulls/5)
	
	return function ()
		-- Destroy Skulls
		Spell:Pause(Spell.SPELL_INTRO_PAUSE)
		Spell:ExplodeGemByName(params, "Skull", false, Spell.SPELL_TRANSFORM_PAUSE)
		Spell:ExplodeGemByName(params, "Skull5", false, Spell.SPELL_TRANSFORM_PAUSE)
		-- Stun Enemy
		Spell:ApplyStatusEffectTo(params.target, params, self, "Stun", 2*numTurns)
		Spell:AwardBonusMove(params)
		Spell:Pause(Spell.SPELL_MIN_DURATION)
		Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	end
end

function SkullCrusher:Notify(grid,notify)
	local heading, text, startTime = Spell:ShowStandardSpellHeader(self.title)
	
	local gems = grid:GetAllGemsByName("Skull")
	table.join(gems,grid:GetAllGemsByName("Skull5"))
	
	local width
	for i,coord in ipairs(gems) do
		local gem = grid:GemAt(coord.x,coord.y)		
		if not width then
			width = gem:GetView():GetExtentsX()
		end
		local size = 1.2 * (width/72)
		local duration = Spell.SPELL_INTRO_PAUSE+i*Spell.SPELL_TRANSFORM_PAUSE
		
		local path = MovementManager:Construct("RoundedNonuniformSpline")
		local x = gem:GetX()
		local y = gem:GetY()
		Spell:PushPosition(path,x,y+1)
		Spell:PushVelocity(path,0,0)
		Spell:PushPosition(path,x,y-1)
		Spell:PushVelocity(path,0,0)
		
		local particle = ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription("FearSkulls"))
		particle:SetOffset(0,0,100)
		particle:SetColor(0.8,0.2,0.2,0.0)
		particle:SetTargetColor(0.8,0.2,0.2,0.3)
		particle:SetNumParticlesToRelease(10)
		particle:SetMaxParticles(10)
		particle:SetLifetime(duration)
		particle:SetLifeCycle(duration)
		particle:SetVelocityVariation(0.05)
		particle:SetSize(size)
		particle:SetAnimateSize(false)

		
		Spell:AddParticleTrail(path, particle, 0,0,duration)
	end
	
	Spell:Pause(Spell.SPELL_INTRO_PAUSE)
	Spell:ShowStandardEffectSubtitle(self.subtitles[1], Spell.SPELL_MIN_DURATION)
	Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	Spell:RemoveStandardSpellHeader(heading, text, startTime)
end

return SkullCrusher
