local Slam = {}
local Spell = require("Spells/Spell")

Slam.id   = "SLAM"
Slam.cost = {
	red     = 6,
	black   = 7,
}
Slam.cooldown = 2
Slam.icon = "img_spell_slam"
Slam.short_name	= "SLAM"
Slam.sound	= "sp_slam"
Slam.sound_ds = "snd_sp_cursedark"
Slam.sound_bank_ds = "BANK_SP_CURSEDARK"
Slam.dietfxParticle	= "magicexplosion"
Slam.dietfxTexture	= "fireball"
Slam.is_stat_logged  = true
Slam.effects = { { "[SLAM_EFFECT0_HEAD]", "[SLAM_EFFECT0_BODY]" } }
Slam.title = Spell:GetTitleTag(Slam)
Slam.subtitles = {
	Spell:GetEffectTag(Slam, 0),
}
Slam.cooldownForAI = 4
Slam.cooldown = 4

function Slam:Apply(params)
	return function ()
		Spell:Pause(Spell.SPELL_INTRO_PAUSE)
		Spell:DamageMana(params, "red", 10)
		Spell:Pause(Spell.SPELL_EFFECT_PAUSE)
		Spell:DamageMana(params, "yellow", 10)
		Spell:Pause(Spell.SPELL_EFFECT_PAUSE)
		Spell:DamageMana(params, "blue", 10)
		Spell:Pause(Spell.SPELL_EFFECT_PAUSE)
		Spell:DamageMana(params, "green", 10)
		Spell:Pause(Spell.SPELL_EFFECT_PAUSE)
		Spell:DamageMana(params, "black",10)
		Spell:Pause(Spell.SPELL_MIN_DURATION)
		Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
		Spell:AwardBonusMove(params)
	end
end

function Slam:Notify(grid,notify)
	local heading, text, startTime = Spell:ShowStandardSpellHeader(self.title)
	
	
	Spell:ThrowParticle(notify,{"FireBallRed"},{2,"icon_green",0,64},3,1,1000)
	Spell:ThrowParticle(notify,{"FireBallRed"},{2,"icon_blue",0,64},3,1,1000)
	Spell:ThrowParticle(notify,{"FireBallRed"},{2,"icon_black",0,64},3,1,1000)
	
	-- get widget info
	local client = notify:GetAttribute("client")
	local opClient = Spell:GetOpposingClient(client)	
	--[[]]
	local delay = Spell.SPELL_INTRO_PAUSE
	-- get positions
	local colors = {"Red","Yellow","Blue","Green","Black"}
	for _,color in ipairs(colors) do
		local end_x, end_y = Spell:GetWidgetTargetPosition(opClient, string.format("icon_%s_filling",string.lower(color)))
		
		local path = MovementManager:Construct("RoundedNonuniformSpline")	
		
		Spell:PushPosition(path, end_x, end_y-10)
		Spell:PushVelocity(path,0,0)
		Spell:PushPosition(path, end_x, end_y-11)
		Spell:PushVelocity(path,0,0)
		
		Spell:AttachParticleMotionFragment(path, {ParticleLibrary.CloneDescription(ParticleLibrary.FindDescription(string.format("Jet%s",color)))}, nil,delay)
		delay = delay + Spell.SPELL_EFFECT_PAUSE
	end
	
	Spell:Pause(Spell.SPELL_INTRO_PAUSE)
	Spell:ShowStandardEffectSubtitle(self.subtitles[1], Spell.SPELL_MIN_DURATION + 4 * Spell.SPELL_EFFECT_PAUSE)
	Spell:Pause(Spell.SPELL_OUTRO_PAUSE)
	Spell:RemoveStandardSpellHeader(heading, text, startTime)
end

function Slam:GetScore(params)
	-- if opponent has less than 5 red mana, do not cast this
	local red = params.target.state.red
	local green = params.target.state.green
	local blue = params.target.state.blue
	local yellow = params.target.state.yellow
	local black = params.target.state.black
	
	if red + green + blue + yellow + black < 20 then
		return 0
	end
	
	-- otherwise use the default spell cast chance
end

return Slam
